@@ -55,8 +55,6 @@ struct _GcalCalendarMonitor
   GMainContext       *thread_context;
   GMainContext       *main_context;
 
-  GHashTable         *events; /* gchar* -> GcalEvent* */
-
   GAsyncQueue        *messages;
   GcalCalendar       *calendar;
   gboolean            complete;
@@ -76,7 +74,8 @@ struct _GcalCalendarMonitor
    * a locked muxed.
    */
   struct {
-    GMutex            mutex;
+    GRWLock           lock;
+    GHashTable       *events; /* gchar* -> GcalEvent* */
     GcalRange        *range;
     gchar            *filter;
   } shared;
@@ -178,9 +177,9 @@ get_monitor_ranges (GcalCalendarMonitor  *self)
 {
   g_autoptr (GcalRange) range = NULL;
 
-  g_mutex_lock (&self->shared.mutex);
+  g_rw_lock_reader_lock (&self->shared.lock);
   range = gcal_range_copy (self->shared.range);
-  g_mutex_unlock (&self->shared.mutex);
+  g_rw_lock_reader_unlock (&self->shared.lock);
 
   return g_steal_pointer (&range);
 }
@@ -326,7 +325,6 @@ on_client_view_objects_added_cb (ECalClientView      *view,
     {
       g_autoptr (GcalEvent) event = NULL;
       g_autoptr (GError) error = NULL;
-      g_autofree gchar *event_id = NULL;
       ICalComponent *icomponent;
       ECalComponent *ecomponent;
 
@@ -363,10 +361,12 @@ on_client_view_objects_added_cb (ECalClientView      *view,
           continue;
         }
 
-      event_id = g_strdup (gcal_event_get_uid (event));
-
       if (!self->monitor_thread.populated)
         {
+          g_autofree gchar *event_id = NULL;
+
+          event_id = g_strdup (gcal_event_get_uid (event));
+
           g_hash_table_insert (self->monitor_thread.events_to_add,
                                g_steal_pointer (&event_id),
                                g_object_ref (event));
@@ -443,16 +443,17 @@ on_client_view_objects_added_cb (ECalClientView      *view,
       /* Process all these instances */
       for (i = 0; i < expanded_events->len; i++)
         {
-          g_autofree gchar *event_id = NULL;
           GcalEvent *event = g_ptr_array_index (expanded_events, i);
 
           if (g_cancellable_is_cancelled (self->cancellable))
             return;
 
-          event_id = g_strdup (gcal_event_get_uid (event));
-
           if (!self->monitor_thread.populated)
             {
+              g_autofree gchar *event_id = NULL;
+
+              event_id = g_strdup (gcal_event_get_uid (event));
+
               g_hash_table_insert (self->monitor_thread.events_to_add,
                                    g_steal_pointer (&event_id),
                                    g_object_ref (event));
@@ -472,7 +473,13 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
                                     const GSList        *objects,
                                     GcalCalendarMonitor *self)
 {
+  g_autoptr (GRWLockReaderLocker) reader_locker = NULL;
+  g_autoptr (GHashTable) events_to_remove = NULL;
+  g_autoptr (GPtrArray) components_to_expand = NULL;
+  g_autoptr (GcalRange) range = NULL;
+  GHashTableIter iter;
   const GSList *l;
+  const gchar *aux;
 
   GCAL_ENTRY;
 
@@ -482,12 +489,20 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
       return;
     }
 
+  reader_locker = g_rw_lock_reader_locker_new (&self->shared.lock);
+  components_to_expand = g_ptr_array_new ();
+  events_to_remove = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+  range = gcal_range_copy (self->shared.range);
+
   for (l = objects; l; l = l->next)
     {
       g_autoptr (GcalEvent) event = NULL;
       g_autoptr (GError) error = NULL;
+      g_autofree gchar *event_id = NULL;
+      ECalComponentId *component_id;
       ICalComponent *icomponent;
       ECalComponent *ecomponent;
+      gboolean recurrence_main;
 
       icomponent = l->data;
 
@@ -499,6 +514,45 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
       if (!ecomponent)
         continue;
 
+      recurrence_main = e_cal_util_component_has_recurrences (icomponent) &&
+                        !e_cal_util_component_is_instance (icomponent);
+
+      /*
+       * If the event has no recurrence id, it is either a main event, or a
+       * regular non-recurring event. If we're receiving the 'objects-modified'
+       * signal because this event went from recurring to non-recurring, the
+       * instances of the old recurrency are still here. They need to be removed.
+       */
+      component_id = e_cal_component_get_id (ecomponent);
+
+      event_id = g_strdup_printf ("%s:%s",
+                                  gcal_calendar_get_id (self->calendar),
+                                  e_cal_component_id_get_uid (component_id));
+
+      if (!e_cal_component_id_get_rid (component_id))
+        {
+          g_hash_table_iter_init (&iter, self->shared.events);
+          while (g_hash_table_iter_next (&iter, (gpointer*) &aux, NULL))
+            {
+              if (!g_str_has_prefix (aux, event_id))
+                continue;
+
+              if (recurrence_main || !g_str_equal (aux, event_id))
+                g_hash_table_add (events_to_remove, g_strdup (aux));
+            }
+        }
+      g_clear_pointer (&component_id, e_cal_component_id_free);
+
+      /* Recurrent events will be processed later */
+      if (recurrence_main)
+        {
+          GCAL_TRACE_MSG ("Component %s (%s) needs to be expanded",
+                          i_cal_component_get_summary (icomponent),
+                          i_cal_component_get_uid (icomponent));
+          g_ptr_array_add (components_to_expand, icomponent);
+          continue;
+        }
+
       event = gcal_event_new (self->calendar, ecomponent, &error);
       g_clear_object (&ecomponent);
 
@@ -511,6 +565,95 @@ on_client_view_objects_modified_cb (ECalClientView      *view,
       update_event_in_idle (self, event);
     }
 
+  /* Recurrent events */
+  if (components_to_expand->len > 0)
+    {
+      g_autoptr (GPtrArray) expanded_events = NULL;
+      g_autoptr (GDateTime) range_start = NULL;
+      g_autoptr (GDateTime) range_end = NULL;
+      ECalClient *client;
+      time_t range_start_time;
+      time_t range_end_time;
+
+      GCAL_TRACE_MSG ("Expanding recurrencies of %d events", components_to_expand->len);
+
+      client = gcal_calendar_get_client (self->calendar);
+
+      range_start = gcal_range_get_start (range);
+      range_end = gcal_range_get_end (range);
+      range_start_time = g_date_time_to_unix (range_start);
+      range_end_time = g_date_time_to_unix (range_end) - 1;
+
+      expanded_events = g_ptr_array_new_with_free_func (g_object_unref);
+
+      /* Generate the instances */
+      for (guint i = 0; i < components_to_expand->len; i++)
+        {
+          GenerateRecurrencesData recurrences_data;
+          ICalComponent *icomponent;
+#if GCAL_ENABLE_TRACE
+          gint old_size;
+#endif
+
+          if (g_cancellable_is_cancelled (self->cancellable))
+            return;
+
+          icomponent = g_ptr_array_index (components_to_expand, i);
+
+          recurrences_data.monitor = self;
+          recurrences_data.expanded_events = expanded_events;
+
+#if GCAL_ENABLE_TRACE
+          old_size = expanded_events->len;
+#endif
+
+          e_cal_client_generate_instances_for_object_sync (client,
+                                                           icomponent,
+                                                           range_start_time,
+                                                           range_end_time,
+                                                           self->cancellable,
+                                                           client_instance_generated_cb,
+                                                           &recurrences_data);
+
+#if GCAL_ENABLE_TRACE
+            {
+              g_autofree gchar *range_str = gcal_range_to_string (range);
+
+              GCAL_TRACE_MSG ("Component %s (%s) added %d instance(s) between %s",
+                              i_cal_component_get_summary (icomponent),
+                              i_cal_component_get_uid (icomponent),
+                              expanded_events->len - old_size,
+                              range_str);
+            }
+#endif
+        }
+
+      for (guint i = 0; i < expanded_events->len; i++)
+        {
+          GcalEvent *event = g_ptr_array_index (expanded_events, i);
+          GcalEvent *old_event;
+
+          if (g_cancellable_is_cancelled (self->cancellable))
+            return;
+
+          old_event = g_hash_table_lookup (self->shared.events, gcal_event_get_uid (event));
+          if (old_event)
+            {
+              update_event_in_idle (self, event);
+              g_hash_table_remove (events_to_remove, gcal_event_get_uid (event));
+            }
+          else
+            {
+              add_event_in_idle (self, event);
+            }
+        }
+    }
+
+  /* Now remove lingering events */
+  g_hash_table_iter_init (&iter, events_to_remove);
+  while (g_hash_table_iter_next (&iter, (gpointer*) &aux, NULL))
+    remove_event_in_idle (self, aux);
+
   GCAL_EXIT;
 }
 
@@ -542,9 +685,26 @@ on_client_view_objects_removed_cb (ECalClientView      *view,
         }
       else
         {
+          g_autoptr (GRWLockReaderLocker) reader_locker = NULL;
+          GHashTableIter iter;
+          const gchar *aux;
+
           event_id = g_strdup_printf ("%s:%s",
                                       gcal_calendar_get_id (self->calendar),
                                       e_cal_component_id_get_uid (component_id));
+
+          /*
+           * If this is the main component, remove the expanded recurrency instances
+           * as well.
+           */
+          reader_locker = g_rw_lock_reader_locker_new (&self->shared.lock);
+
+          g_hash_table_iter_init (&iter, self->shared.events);
+          while (g_hash_table_iter_next (&iter, (gpointer*) &aux, NULL))
+            {
+              if (!g_str_equal (aux, event_id) && g_str_has_prefix (aux, event_id))
+                remove_event_in_idle (self, aux);
+            }
         }
 
       remove_event_in_idle (self, event_id);
@@ -598,6 +758,7 @@ on_client_view_complete_cb (ECalClientView      *view,
 static void
 create_view (GcalCalendarMonitor *self)
 {
+  g_autoptr (GRWLockReaderLocker) reader_locker = NULL;
   g_autofree gchar *filter = NULL;
   g_autoptr (GError) error = NULL;
   ECalClientView *view;
@@ -605,20 +766,17 @@ create_view (GcalCalendarMonitor *self)
 
   GCAL_ENTRY;
 
-  g_mutex_lock (&self->shared.mutex);
+  reader_locker = g_rw_lock_reader_locker_new (&self->shared.lock);
 
   g_assert (self->cancellable == NULL);
   self->cancellable = g_cancellable_new ();
 
   if (!self->shared.range)
-    {
-      g_mutex_unlock (&self->shared.mutex);
-      GCAL_RETURN ();
-    }
+    GCAL_RETURN ();
 
   filter = build_subscriber_filter (self->shared.range, self->shared.filter);
 
-  g_mutex_unlock (&self->shared.mutex);
+  g_clear_pointer (&reader_locker, g_rw_lock_reader_locker_free);
 
   if (!gcal_calendar_get_visible (self->calendar))
     GCAL_RETURN ();
@@ -830,12 +988,15 @@ static void
 remove_events_outside_range (GcalCalendarMonitor *self,
                              GcalRange           *range)
 {
+  g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
   GHashTableIter iter;
   GcalEvent *event;
 
   GCAL_TRACE_MSG ("Removing events outside range from monitor");
 
-  g_hash_table_iter_init (&iter, self->events);
+  writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
+
+  g_hash_table_iter_init (&iter, self->shared.events);
   while (g_hash_table_iter_next (&iter, NULL, (gpointer*) &event))
     {
       g_object_ref (event);
@@ -850,12 +1011,15 @@ remove_events_outside_range (GcalCalendarMonitor *self,
 static void
 remove_all_events (GcalCalendarMonitor *self)
 {
+  g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
   GHashTableIter iter;
   GcalEvent *event;
 
   GCAL_TRACE_MSG ("Removing all events from view");
 
-  g_hash_table_iter_init (&iter, self->events);
+  writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
+
+  g_hash_table_iter_init (&iter, self->shared.events);
   while (g_hash_table_iter_next (&iter, NULL, (gpointer*) &event))
     {
       g_hash_table_iter_remove (&iter);
@@ -884,6 +1048,7 @@ set_complete (GcalCalendarMonitor *self,
 static gboolean
 add_event_to_timeline_in_idle_cb (gpointer user_data)
 {
+  g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
   GcalCalendarMonitor *self;
   GcalEvent *event;
   IdleData *idle_data;
@@ -897,7 +1062,9 @@ add_event_to_timeline_in_idle_cb (gpointer user_data)
   g_assert (idle_data->event_id == NULL);
 
   event_id = gcal_event_get_uid (event);
-  if (g_hash_table_insert (self->events, g_strdup (event_id), g_object_ref (event)))
+
+  writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
+  if (g_hash_table_insert (self->shared.events, g_strdup (event_id), g_object_ref (event)))
     g_signal_emit (self, signals[EVENT_ADDED], 0, event);
 
   GCAL_RETURN (G_SOURCE_REMOVE);
@@ -906,6 +1073,7 @@ add_event_to_timeline_in_idle_cb (gpointer user_data)
 static gboolean
 update_event_in_idle_cb (gpointer user_data)
 {
+  g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
   GcalCalendarMonitor *self;
   GcalEvent *old_event;
   GcalEvent *event;
@@ -920,10 +1088,12 @@ update_event_in_idle_cb (gpointer user_data)
   g_assert (idle_data->event_id == NULL);
 
   event_id = gcal_event_get_uid (event);
-  old_event = g_hash_table_lookup (self->events, event_id);
+
+  writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
+  old_event = g_hash_table_lookup (self->shared.events, event_id);
   if (old_event)
     {
-      g_hash_table_insert (self->events, g_strdup (event_id), g_object_ref (event));
+      g_hash_table_insert (self->shared.events, g_strdup (event_id), g_object_ref (event));
       g_signal_emit (self, signals[EVENT_UPDATED], 0, old_event, event);
     }
 
@@ -933,6 +1103,7 @@ update_event_in_idle_cb (gpointer user_data)
 static gboolean
 remove_event_from_timeline_in_idle_cb (gpointer user_data)
 {
+  g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
   g_autoptr (GcalEvent) event = NULL;
   GcalCalendarMonitor *self;
   IdleData *idle_data;
@@ -945,14 +1116,15 @@ remove_event_from_timeline_in_idle_cb (gpointer user_data)
   g_assert (idle_data->event == NULL);
   event_id = idle_data->event_id;
 
-  event = g_hash_table_lookup (self->events, event_id);
+  writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
+  event = g_hash_table_lookup (self->shared.events, event_id);
 
   if (event)
     {
       /* Keep the event alive until the signal is emitted */
       g_object_ref (event);
 
-      g_hash_table_remove (self->events, event_id);
+      g_hash_table_remove (self->shared.events, event_id);
       g_signal_emit (self, signals[EVENT_REMOVED], 0, event);
     }
 
@@ -1010,10 +1182,10 @@ gcal_calendar_monitor_finalize (GObject *object)
   remove_all_events (self);
 
   g_clear_object (&self->calendar);
-  g_clear_pointer (&self->events, g_hash_table_destroy);
   g_clear_pointer (&self->thread_context, g_main_context_unref);
   g_clear_pointer (&self->main_context, g_main_context_unref);
   g_clear_pointer (&self->messages, g_async_queue_unref);
+  g_clear_pointer (&self->shared.events, g_hash_table_destroy);
   g_clear_pointer (&self->shared.filter, g_free);
   g_clear_pointer (&self->shared.range, gcal_range_unref);
 
@@ -1121,11 +1293,13 @@ gcal_calendar_monitor_class_init (GcalCalendarMonitorClass *klass)
 static void
 gcal_calendar_monitor_init (GcalCalendarMonitor *self)
 {
-  self->events = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+  self->shared.events = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
   self->thread_context = g_main_context_new ();
   self->main_context = g_main_context_ref_thread_default ();
   self->messages = g_async_queue_new ();
   self->complete = FALSE;
+
+  g_rw_lock_init (&self->shared.lock);
 }
 
 GcalCalendarMonitor*
@@ -1149,13 +1323,14 @@ void
 gcal_calendar_monitor_set_range (GcalCalendarMonitor *self,
                                  GcalRange           *range)
 {
+  g_autoptr (GRWLockWriterLocker) writer_locker = NULL;
   gboolean range_changed;
 
   g_return_if_fail (GCAL_IS_CALENDAR_MONITOR (self));
 
   GCAL_ENTRY;
 
-  g_mutex_lock (&self->shared.mutex);
+  writer_locker = g_rw_lock_writer_locker_new (&self->shared.lock);
 
   range_changed =
     !self->shared.range ||
@@ -1163,15 +1338,12 @@ gcal_calendar_monitor_set_range (GcalCalendarMonitor *self,
     gcal_range_calculate_overlap (self->shared.range, range, NULL) != GCAL_RANGE_EQUAL;
 
   if (!range_changed)
-    {
-      g_mutex_unlock (&self->shared.mutex);
-      GCAL_RETURN ();
-    }
+    GCAL_RETURN ();
 
   g_clear_pointer (&self->shared.range, gcal_range_unref);
   self->shared.range = range ? gcal_range_copy (range) : NULL;
 
-  g_mutex_unlock (&self->shared.mutex);
+  g_clear_pointer (&writer_locker, g_rw_lock_writer_locker_free);
 
   maybe_spawn_view_thread (self);
   remove_events_outside_range (self, range);
@@ -1200,7 +1372,9 @@ gcal_calendar_monitor_get_cached_event (GcalCalendarMonitor *self,
   g_return_val_if_fail (GCAL_IS_CALENDAR_MONITOR (self), NULL);
   g_return_val_if_fail (event_id, NULL);
 
-  event = g_hash_table_lookup (self->events, event_id);
+  g_rw_lock_reader_lock (&self->shared.lock);
+  event = g_hash_table_lookup (self->shared.events, event_id);
+  g_rw_lock_reader_unlock (&self->shared.lock);
 
   return event ? g_object_ref (event) : NULL;
 }
@@ -1211,12 +1385,12 @@ gcal_calendar_monitor_set_filter (GcalCalendarMonitor *self,
 {
   g_return_if_fail (GCAL_IS_CALENDAR_MONITOR (self));
 
-  g_mutex_lock (&self->shared.mutex);
+  g_rw_lock_writer_lock (&self->shared.lock);
 
   g_clear_pointer (&self->shared.filter, g_free);
   self->shared.filter = g_strdup (filter);
 
-  g_mutex_unlock (&self->shared.mutex);
+  g_rw_lock_writer_unlock (&self->shared.lock);
 
   remove_all_events (self);
 
