@@ -31,7 +31,6 @@
 #include "gcal-schedule-section.h"
 #include "gcal-summary-section.h"
 
-#include <dazzle.h>
 #include <libecal/libecal.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -58,9 +57,8 @@ struct _GcalEventEditorDialog
   GcalEventEditorSection *notes_section;
   GcalEventEditorSection *reminders_section;
   GcalEventEditorSection *schedule_section;
-  GtkWidget              *scrolled_window;
   GtkWidget              *sources_button;
-  GtkWidget              *sources_popover;
+  GtkPopoverMenu         *sources_popover;
   GtkWidget              *source_image;
   GtkWidget              *source_label;
   GtkWidget              *subtitle_label;
@@ -152,40 +150,37 @@ fill_sources_menu (GcalEventEditorDialog *self)
 
   for (aux = list; aux != NULL; aux = aux->next)
     {
+      g_autoptr (GdkPaintable) paintable = NULL;
       g_autoptr (GMenuItem) item = NULL;
       g_autoptr (GdkPixbuf) pix = NULL;
+      g_autoptr (GVariant) target = NULL;
       GcalCalendar *calendar;
       const GdkRGBA *color;
-      cairo_surface_t *surface;
 
       calendar = GCAL_CALENDAR (aux->data);
 
       /* retrieve color */
       color = gcal_calendar_get_color (calendar);
-      surface = get_circle_surface_from_color (color, 16);
-      pix = gdk_pixbuf_get_from_surface (surface, 0, 0, 16, 16);
+      paintable = get_circle_paintable_from_color (color, 16);
+      pix = paintable_to_pixbuf (paintable);
 
       /* menu item */
       item = g_menu_item_new (gcal_calendar_get_name (calendar), "select-calendar");
       g_menu_item_set_icon (item, G_ICON (pix));
 
       /* set insensitive for read-only calendars */
-      if (gcal_calendar_is_read_only (calendar))
-        {
-          g_menu_item_set_action_and_target_value (item, "select-calendar", NULL);
-        }
-      else
-        {
-          const gchar *id = gcal_calendar_get_id (calendar);
-          g_menu_item_set_action_and_target_value (item, "select-calendar", g_variant_new_string (id));
-        }
+      if (!gcal_calendar_is_read_only (calendar))
+        target = g_variant_new_string (gcal_calendar_get_id (calendar));
+
+      g_menu_item_set_action_and_target_value (item,
+                                               "event-editor-dialog.select-calendar",
+                                               g_steal_pointer (&target));
 
       g_menu_append_item (self->sources_menu, item);
-
-      g_clear_pointer (&surface, cairo_surface_destroy);
     }
 
-  gtk_popover_bind_model (GTK_POPOVER (self->sources_popover), G_MENU_MODEL (self->sources_menu), "edit");
+  gtk_popover_menu_set_menu_model (self->sources_popover,
+                                   G_MENU_MODEL (self->sources_menu));
 
   /* HACK: show the popover menu icons */
   fix_popover_menu_icons (GTK_POPOVER (self->sources_popover));
@@ -212,6 +207,64 @@ clear_and_hide_dialog (GcalEventEditorDialog *self)
   gtk_widget_hide (GTK_WIDGET (self));
 }
 
+static void
+apply_event_properties_to_template_event (GcalEvent *template_event,
+                                          GcalEvent *event)
+{
+  g_autoptr (GDateTime) start_date = NULL;
+  g_autoptr (GDateTime) end_date = NULL;
+  GDateTime *template_start_date;
+  GDateTime *event_start_date;
+  GDateTime *template_end_date;
+  GDateTime *event_end_date;
+  gboolean was_all_day;
+  gboolean is_all_day;
+
+  is_all_day = gcal_event_get_all_day (event);
+  was_all_day = gcal_event_get_all_day (template_event);
+
+  template_start_date = gcal_event_get_date_start (template_event);
+  template_end_date = gcal_event_get_date_end (template_event);
+  event_start_date = gcal_event_get_date_start (event);
+  event_end_date = gcal_event_get_date_end (event);
+
+  start_date = g_date_time_new (g_date_time_get_timezone (event_start_date),
+                                g_date_time_get_year (template_start_date),
+                                g_date_time_get_month (template_start_date),
+                                g_date_time_get_day_of_month (template_start_date),
+                                g_date_time_get_hour (event_start_date),
+                                g_date_time_get_minute (event_start_date),
+                                g_date_time_get_second (event_start_date));
+
+  end_date = g_date_time_new (g_date_time_get_timezone (event_end_date),
+                              g_date_time_get_year (template_end_date),
+                              g_date_time_get_month (template_end_date),
+                              g_date_time_get_day_of_month (template_end_date),
+                              g_date_time_get_hour (event_end_date),
+                              g_date_time_get_minute (event_end_date),
+                              g_date_time_get_second (event_end_date));
+
+  if (was_all_day != is_all_day)
+    {
+      g_autoptr (GDateTime) fake_end_date = NULL;
+
+      if (is_all_day)
+        fake_end_date = g_date_time_add_days (end_date, -1);
+      else
+        fake_end_date = g_date_time_add_days (end_date, 1);
+
+      gcal_set_date_time (&end_date, fake_end_date);
+    }
+
+  gcal_event_set_summary (template_event, gcal_event_get_summary (event));
+  gcal_event_set_location (template_event, gcal_event_get_location (event));
+  gcal_event_set_description (template_event, gcal_event_get_description (event));
+  gcal_event_set_recurrence (template_event, gcal_event_get_recurrence (event));
+  gcal_event_set_all_day (template_event, gcal_event_get_all_day (event));
+  gcal_event_set_date_start (template_event, start_date);
+  gcal_event_set_date_end (template_event, end_date);
+}
+
 
 /*
  * Callbacks
@@ -244,19 +297,17 @@ on_calendar_selected_action_cb (GSimpleAction *action,
 
       if (g_strcmp0 (gcal_calendar_get_id (calendar), uid) == 0)
         {
-          cairo_surface_t *surface;
+          g_autoptr (GdkPaintable) paintable = NULL;
           const GdkRGBA *color;
 
           /* retrieve color */
           color = gcal_calendar_get_color (calendar);
-          surface = get_circle_surface_from_color (color, 16);
-          gtk_image_set_from_surface (GTK_IMAGE (self->source_image), surface);
+          paintable = get_circle_paintable_from_color (color, 16);
+          gtk_image_set_from_paintable (GTK_IMAGE (self->source_image), paintable);
 
           self->selected_calendar = calendar;
 
           gtk_label_set_label (GTK_LABEL (self->subtitle_label), gcal_calendar_get_name (calendar));
-
-          g_clear_pointer (&surface, cairo_surface_destroy);
           break;
         }
     }
@@ -264,19 +315,6 @@ on_calendar_selected_action_cb (GSimpleAction *action,
   GCAL_EXIT;
 }
 
-static void
-transient_size_allocate_cb (GcalEventEditorDialog *self)
-{
-  GtkAllocation alloc;
-  GtkWindow *transient;
-
-  transient = gtk_window_get_transient_for (GTK_WINDOW (self));
-  gtk_widget_get_allocation (GTK_WIDGET (transient), &alloc);
-
-  gtk_scrolled_window_set_max_content_height (GTK_SCROLLED_WINDOW (self->scrolled_window),
-                                              MAX (400, (gint) (0.75 * alloc.height)));
-}
-
 static void
 on_cancel_button_clicked_cb (GtkButton             *button,
                              GcalEventEditorDialog *self)
@@ -288,20 +326,110 @@ on_cancel_button_clicked_cb (GtkButton             *button,
   GCAL_EXIT;
 }
 
+static void
+on_ask_recurrence_response_delete_cb (GcalEvent             *event,
+                                      GcalRecurrenceModType  mod_type,
+                                      gpointer               user_data)
+{
+  GcalEventEditorDialog *self = GCAL_EVENT_EDITOR_DIALOG (user_data);
+
+  if (mod_type == GCAL_RECURRENCE_MOD_NONE)
+    return;
+
+  g_signal_emit (self, signals[REMOVE_EVENT], 0, event, mod_type);
+  clear_and_hide_dialog (self);
+}
+
 static void
 on_delete_button_clicked_cb (GtkButton             *button,
                              GcalEventEditorDialog *self)
 {
   GcalRecurrenceModType mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
-  GcalCalendar *calendar;
 
   GCAL_ENTRY;
 
-  calendar = gcal_event_get_calendar (self->event);
-  if (!gcal_event_has_recurrence (self->event) ||
-      ask_recurrence_modification_type (GTK_WIDGET (self), &mod, calendar))
+  if (gcal_event_has_recurrence (self->event))
+    {
+      gcal_utils_ask_recurrence_modification_type (GTK_WIDGET (self),
+                                                   self->event,
+                                                   TRUE,
+                                                   on_ask_recurrence_response_delete_cb,
+                                                   self);
+    }
+  else
     {
       g_signal_emit (self, signals[REMOVE_EVENT], 0, self->event, mod);
+      clear_and_hide_dialog (self);
+    }
+
+  GCAL_EXIT;
+}
+
+static void
+on_ask_recurrence_response_save_cb (GcalEvent             *event,
+                                    GcalRecurrenceModType  mod_type,
+                                    gpointer               user_data)
+{
+  GcalEventEditorDialog *self = GCAL_EVENT_EDITOR_DIALOG (user_data);
+  GcalManager *manager;
+
+  GCAL_ENTRY;
+
+  manager = gcal_context_get_manager (self->context);
+
+  switch (mod_type)
+    {
+    case GCAL_RECURRENCE_MOD_NONE:
+      GCAL_RETURN ();
+
+    case GCAL_RECURRENCE_MOD_ALL:
+      {
+        g_autoptr (GcalEvent) template_event = NULL;
+        g_autoptr (GError) error = NULL;
+        ECalComponentId *component_id;
+        ICalComponent *template_icomponent;
+        ECalComponent *template_ecomponent;
+        ECalComponent *component;
+        GcalCalendar *calendar;
+        ECalClient *client;
+
+        calendar = gcal_event_get_calendar (event);
+        client = gcal_calendar_get_client (calendar);
+        component = gcal_event_get_component (event);
+        component_id = e_cal_component_get_id (component);
+
+        e_cal_client_get_object_sync (client,
+                                      e_cal_component_id_get_uid (component_id),
+                                      NULL,
+                                      &template_icomponent,
+                                      NULL,
+                                      &error);
+
+        g_clear_pointer (&component_id, e_cal_component_id_free);
+
+        if (error)
+          {
+            g_warning ("Error updating event: %s", error->message);
+            break;
+          }
+
+        template_ecomponent = e_cal_component_new_from_icalcomponent (template_icomponent);
+        template_event = gcal_event_new (calendar, template_ecomponent, &error);
+        if (error)
+          {
+            g_warning ("Error updating event: %s", error->message);
+            break;
+          }
+
+        apply_event_properties_to_template_event (template_event, event);
+        gcal_manager_update_event (manager, template_event, mod_type);
+      }
+      break;
+
+    case GCAL_RECURRENCE_MOD_THIS_AND_FUTURE:
+    case GCAL_RECURRENCE_MOD_THIS_ONLY:
+      gcal_manager_update_event (manager, self->event, mod_type);
+      break;
     }
 
   clear_and_hide_dialog (self);
@@ -316,17 +444,46 @@ on_done_button_clicked_cb (GtkButton             *button,
   GcalCalendar *selected_calendar;
   GcalCalendar *calendar;
   GcalManager *manager;
+  gboolean can_show_mod_all;
+  gboolean was_recurrent;
   gint i;
 
-  for (i = 0; i < G_N_ELEMENTS (self->sections); i++)
-    gcal_event_editor_section_apply (self->sections[i]);
-
   manager = gcal_context_get_manager (self->context);
   calendar = gcal_event_get_calendar (self->event);
 
   if (gcal_calendar_is_read_only (calendar))
     GCAL_GOTO (out);
 
+  can_show_mod_all = TRUE;
+  if (!self->event_is_new)
+    {
+      gboolean anything_changed = FALSE;
+
+      for (i = 0; i < G_N_ELEMENTS (self->sections); i++)
+        {
+          gboolean section_changed;
+
+          section_changed = gcal_event_editor_section_changed (self->sections[i]);
+          anything_changed |= section_changed;
+        }
+
+      if (!anything_changed)
+        goto out;
+
+      can_show_mod_all =
+        !gcal_schedule_section_recurrence_changed (GCAL_SCHEDULE_SECTION (self->schedule_section)) &&
+        !gcal_schedule_section_day_changed (GCAL_SCHEDULE_SECTION (self->schedule_section));
+    }
+
+  /*
+   * We don't want to ask the recurrence mod type if the event wasn't
+   * actually recurrent.
+   */
+  was_recurrent = gcal_event_has_recurrence (self->event);
+
+  for (i = 0; i < G_N_ELEMENTS (self->sections); i++)
+    gcal_event_editor_section_apply (self->sections[i]);
+
   selected_calendar = g_steal_pointer (&self->selected_calendar);
   if (selected_calendar && calendar != selected_calendar)
     {
@@ -347,17 +504,18 @@ on_done_button_clicked_cb (GtkButton             *button,
     {
       gcal_manager_create_event (manager, self->event);
     }
+  else if (was_recurrent && gcal_event_has_recurrence (self->event))
+    {
+      gcal_utils_ask_recurrence_modification_type (GTK_WIDGET (self),
+                                                   self->event,
+                                                   can_show_mod_all,
+                                                   on_ask_recurrence_response_save_cb,
+                                                   self);
+      return;
+    }
   else
     {
-      GcalRecurrenceModType mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
-
-      if (gcal_event_has_recurrence (self->event) &&
-          !ask_recurrence_modification_type (GTK_WIDGET (self), &mod, calendar))
-        {
-          goto out;
-        }
-
-      gcal_manager_update_event (manager, self->event, mod);
+      gcal_manager_update_event (manager, self->event, GCAL_RECURRENCE_MOD_THIS_ONLY);
     }
 
 out:
@@ -410,15 +568,8 @@ gcal_event_editor_dialog_constructed (GObject* object)
                                    self);
 
   gtk_widget_insert_action_group (GTK_WIDGET (self),
-                                  "edit",
+                                  "event-editor-dialog",
                                   G_ACTION_GROUP (self->action_group));
-
-  /* Watch the main window and adapt the maximum size */
-  g_signal_connect_object (gtk_window_get_transient_for (GTK_WINDOW (self)),
-                           "size-allocate",
-                           G_CALLBACK (transient_size_allocate_cb),
-                           self,
-                           G_CONNECT_SWAPPED);
 }
 
 static void
@@ -542,7 +693,6 @@ gcal_event_editor_dialog_class_init (GcalEventEditorDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalEventEditorDialog, notes_section);
   gtk_widget_class_bind_template_child (widget_class, GcalEventEditorDialog, reminders_section);
   gtk_widget_class_bind_template_child (widget_class, GcalEventEditorDialog, schedule_section);
-  gtk_widget_class_bind_template_child (widget_class, GcalEventEditorDialog, scrolled_window);
   gtk_widget_class_bind_template_child (widget_class, GcalEventEditorDialog, sources_button);
   gtk_widget_class_bind_template_child (widget_class, GcalEventEditorDialog, sources_popover);
   gtk_widget_class_bind_template_child (widget_class, GcalEventEditorDialog, source_image);
@@ -602,10 +752,10 @@ gcal_event_editor_dialog_set_event (GcalEventEditorDialog *self,
                                     GcalEvent             *event,
                                     gboolean               new_event)
 {
+  g_autoptr (GdkPaintable) paintable = NULL;
   g_autoptr (GcalEvent) cloned_event = NULL;
   GcalEventEditorFlags flags;
   GcalCalendar *calendar;
-  cairo_surface_t *surface;
   gint i;
 
   GCAL_ENTRY;
@@ -630,9 +780,8 @@ gcal_event_editor_dialog_set_event (GcalEventEditorDialog *self,
   fill_sources_menu (self);
 
   /* dialog titlebar's title & subtitle */
-  surface = get_circle_surface_from_color (gcal_event_get_color (cloned_event), 10);
-  gtk_image_set_from_surface (GTK_IMAGE (self->source_image), surface);
-  g_clear_pointer (&surface, cairo_surface_destroy);
+  paintable = get_circle_paintable_from_color (gcal_event_get_color (cloned_event), 10);
+  gtk_image_set_from_paintable (GTK_IMAGE (self->source_image), paintable);
 
   g_clear_pointer (&self->event_title_binding, g_binding_unbind);
   self->event_title_binding = g_object_bind_property (cloned_event,
