@@ -233,14 +233,9 @@ format_datetime_for_display (GDateTime      *date,
 
   switch (days_diff)
     {
-    case -7:
-    case -6:
-    case -5:
-    case -4:
-    case -3:
-    case -2:
+    case -7 ... -2:
       /* Translators: %A is the weekday name (e.g. Sunday, Monday, etc) */
-      formatted_date = g_date_time_format (local_dt, _("Last %A"));
+      formatted_date = g_date_time_format (local_dt, gcal_util_translate_time_string (_("Last %A")));
       break;
 
     case -1:
@@ -255,14 +250,9 @@ format_datetime_for_display (GDateTime      *date,
       formatted_date = g_strdup (_("Tomorrow"));
       break;
 
-    case 2:
-    case 3:
-    case 4:
-    case 5:
-    case 6:
-    case 7:
+    case 2 ... 7:
       /* Translators: %A is the weekday name (e.g. Sunday, Monday, etc) */
-      formatted_date = g_date_time_format (local_dt, _("This %A"));
+      formatted_date = g_date_time_format (local_dt, gcal_util_translate_time_string (_("This %A")));
       break;
 
     default:
@@ -619,7 +609,7 @@ gcal_schedule_section_apply (GcalEventEditorSection *section)
 
   if (freq != GCAL_RECURRENCE_NO_REPEAT)
     {
-      GcalRecurrence *recur;
+      g_autoptr (GcalRecurrence) recur = NULL;
 
       recur = gcal_recurrence_new ();
       recur->frequency = freq;
@@ -639,8 +629,6 @@ gcal_schedule_section_apply (GcalEventEditorSection *section)
           /* ... and set the new one */
           gcal_event_set_recurrence (self->event, recur);
         }
-
-      g_clear_pointer (&recur, gcal_recurrence_unref);
     }
   else
     {
@@ -651,11 +639,68 @@ gcal_schedule_section_apply (GcalEventEditorSection *section)
   GCAL_EXIT;
 }
 
+static gboolean
+gcal_schedule_section_changed (GcalEventEditorSection *section)
+{
+  g_autoptr (GDateTime) start_date = NULL;
+  g_autoptr (GDateTime) end_date = NULL;
+  GcalScheduleSection *self;
+  gboolean was_all_day;
+  gboolean all_day;
+
+  GCAL_ENTRY;
+
+  self = GCAL_SCHEDULE_SECTION (section);
+  all_day = gtk_switch_get_active (self->all_day_switch);
+  was_all_day = gcal_event_get_all_day (self->event);
+
+  /* All day */
+  if (all_day != was_all_day)
+    GCAL_RETURN (TRUE);
+
+  /* Start date */
+  start_date = get_date_start (self);
+  if (!g_date_time_equal (start_date, gcal_event_get_date_start (self->event)))
+    GCAL_RETURN (TRUE);
+
+  /* End date */
+  end_date = get_date_end (self);
+  if (gtk_switch_get_active (self->all_day_switch))
+    {
+      g_autoptr (GDateTime) fake_end_date = g_date_time_add_days (end_date, 1);
+      gcal_set_date_time (&end_date, fake_end_date);
+    }
+  else if (!all_day && was_all_day)
+    {
+      /* When an all day event is changed to be not an all day event, we
+       * need to correct for the fact that the event's timezone was until
+       * now set to UTC. That means we need to change the timezone to
+       * localtime now, or else it will be saved incorrectly.
+       */
+      GDateTime *localtime_date;
+
+      localtime_date = g_date_time_to_local (start_date);
+      g_clear_pointer (&start_date, g_date_time_unref);
+      start_date = localtime_date;
+
+      localtime_date = g_date_time_to_local (end_date);
+      g_clear_pointer (&end_date, g_date_time_unref);
+      end_date = localtime_date;
+    }
+
+  if (!g_date_time_equal (end_date, gcal_event_get_date_end (self->event)))
+    GCAL_RETURN (TRUE);
+
+  /* Recurrency */
+  GCAL_RETURN (gcal_schedule_section_recurrence_changed (self));
+}
+
 static void
 gcal_event_editor_section_iface_init (GcalEventEditorSectionInterface *iface)
 {
   iface->set_event = gcal_schedule_section_set_event;
   iface->apply = gcal_schedule_section_apply;
+  iface->changed = gcal_schedule_section_changed;
 }
 
 
@@ -759,3 +804,43 @@ gcal_schedule_section_init (GcalScheduleSection *self)
 {
   gtk_widget_init_template (GTK_WIDGET (self));
 }
+
+gboolean
+gcal_schedule_section_recurrence_changed (GcalScheduleSection *self)
+{
+  g_autoptr (GcalRecurrence) recurrence = NULL;
+  GcalRecurrenceFrequency freq;
+
+  g_return_val_if_fail (GCAL_IS_SCHEDULE_SECTION (self), FALSE);
+
+  freq = gtk_combo_box_get_active (GTK_COMBO_BOX (self->repeat_combo));
+  if (freq == GCAL_RECURRENCE_NO_REPEAT && !gcal_event_get_recurrence (self->event))
+    GCAL_RETURN (FALSE);
+
+  recurrence = gcal_recurrence_new ();
+  recurrence->frequency = gtk_combo_box_get_active (GTK_COMBO_BOX (self->repeat_combo));
+  recurrence->limit_type = gtk_combo_box_get_active (GTK_COMBO_BOX (self->repeat_duration_combo));
+  if (recurrence->limit_type == GCAL_RECURRENCE_UNTIL)
+    recurrence->limit.until = gcal_date_selector_get_date (GCAL_DATE_SELECTOR (self->until_date_selector));
+  else if (recurrence->limit_type == GCAL_RECURRENCE_COUNT)
+    recurrence->limit.count = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (self->number_of_occurrences_spin));
+
+  GCAL_RETURN (!gcal_recurrence_is_equal (recurrence, gcal_event_get_recurrence (self->event)));
+}
+
+gboolean
+gcal_schedule_section_day_changed (GcalScheduleSection *self)
+{
+  g_autoptr (GDateTime) start_date = NULL;
+  g_autoptr (GDateTime) end_date = NULL;
+
+  g_return_val_if_fail (GCAL_IS_SCHEDULE_SECTION (self), FALSE);
+
+  GCAL_ENTRY;
+
+  start_date = get_date_start (self);
+  end_date = get_date_end (self);
+
+  GCAL_RETURN (gcal_date_time_compare_date (start_date, gcal_event_get_date_start (self->event)) < 0 ||
+               gcal_date_time_compare_date (end_date, gcal_event_get_date_end (self->event)) > 0);
+}
