@@ -32,11 +32,10 @@
 #include <glib-object.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
-#include <handy.h>
 
 struct _GcalApplication
 {
-  DzlApplication      parent;
+  AdwApplication      parent;
 
   GtkWidget          *window;
 
@@ -48,7 +47,7 @@ struct _GcalApplication
   GcalContext        *context;
 };
 
-G_DEFINE_TYPE (GcalApplication, gcal_application, DZL_TYPE_APPLICATION);
+G_DEFINE_TYPE (GcalApplication, gcal_application, ADW_TYPE_APPLICATION);
 
 static gboolean show_version = FALSE;
 
@@ -135,44 +134,15 @@ gcal_application_launch_search (GSimpleAction *search,
 {
 }
 
-static void
-on_about_response (GtkAboutDialog *about,
-                   int             response_id,
-                   gpointer        user_data)
-{
-  g_return_if_fail (GTK_IS_ABOUT_DIALOG (about));
-  g_return_if_fail (user_data == NULL);
-
-  if (response_id == GTK_RESPONSE_CANCEL)
-    gtk_widget_destroy (GTK_WIDGET (about));
-}
-
 static gchar*
 build_about_copyright (GcalApplication *self)
 {
   g_autoptr (GDateTime) dt = NULL;
-  GcalWeatherService *weather_service;
-  const gchar *attribution;
-  GString *builder;
 
-  builder = g_string_new ("<span size=\"small\">");
   dt = g_date_time_new_now_local ();
 
-  /* Build string: */
-  g_string_append_printf (builder,
-                         _("Copyright \xC2\xA9 2012\xE2\x80\x93%d " "The Calendar authors"),
-                          g_date_time_get_year (dt));
-
-  weather_service = gcal_context_get_weather_service (self->context);
-  attribution = gcal_weather_service_get_attribution (weather_service);
-  if (attribution)
-    {
-      g_string_append_c (builder, '\n');
-      g_string_append (builder, attribution);
-    }
-  g_string_append (builder, "</span>");
-
-  return g_string_free (builder, FALSE);
+  return g_strdup_printf(_("Copyright \xC2\xA9 2012\xE2\x80\x93%d The Calendar authors"),
+                         g_date_time_get_year (dt));
 }
 
 static void
@@ -180,19 +150,18 @@ gcal_application_show_about (GSimpleAction *simple,
                              GVariant      *parameter,
                              gpointer       user_data)
 {
+  GcalWeatherService *weather_service;
   GcalApplication *self;
-  GtkWidget *dialog;
-  GtkLabel *copyright_label;
-  g_autofree gchar *program_name = NULL;
+  GtkWidget *about;
   g_autofree gchar *copyright = NULL;
 
-  const gchar *authors[] = {
+  const gchar *developers[] = {
     "Erick Pérez Castellanos <erickpc@gnome.org>",
     "Georges Basile Stavracas Neto <georges.stavracas@gmail.com>",
     "Isaque Galdino <igaldino@gmail.com>",
     NULL
   };
-  const gchar *artists[] = {
+  const gchar *designers[] = {
     "Jakub Steiner <jimmac@gmail.com>",
     "Lapo Calamandrei <calamandrei@gmail.com>",
     "Reda Lazri <the.red.shortcut@gmail.com>",
@@ -200,32 +169,33 @@ gcal_application_show_about (GSimpleAction *simple,
     NULL
   };
 
-
   self = GCAL_APPLICATION (user_data);
 
-  program_name = g_strconcat (_("Calendar"), NAME_SUFFIX, NULL);
-  dialog = g_object_new (GTK_TYPE_ABOUT_DIALOG,
-                         "transient-for", GTK_WINDOW (self->window),
-                         "modal", TRUE,
-                         "destroy-with-parent", TRUE,
-                         "program-name", program_name,
-                         "version", VERSION,
-                         "license-type", GTK_LICENSE_GPL_3_0,
-                         "authors", authors,
-                         "artists", artists,
-                         "logo-icon-name", APPLICATION_ID,
-                         "translator-credits", _("translator-credits"),
-                         NULL);
-
   copyright = build_about_copyright (self);
-  copyright_label = GTK_LABEL (gtk_widget_get_template_child (GTK_WIDGET (dialog),
-                                                              GTK_TYPE_ABOUT_DIALOG,
-                                                              "copyright_label"));
-  gtk_label_set_markup (copyright_label, copyright);
-  gtk_widget_show (GTK_WIDGET (copyright_label));
 
-  g_signal_connect (dialog, "response", G_CALLBACK (on_about_response), NULL);
-  gtk_widget_show (dialog);
+  about = g_object_new (ADW_TYPE_ABOUT_WINDOW,
+                        "transient-for", GTK_WINDOW (self->window),
+                        "application-name", _("Calendar"),
+                        "application-icon", APPLICATION_ID,
+                        "developer-name", _("The GNOME Project"),
+                        "version", VERSION,
+                        "copyright", copyright,
+                        "website", "https://wiki.gnome.org/Apps/Calendar",
+                        "issue-url", "https://gitlab.gnome.org/GNOME/gnome-calendar/-/issues/new",
+                        "license-type", GTK_LICENSE_GPL_3_0,
+                        "developers", developers,
+                        "designers", designers,
+                        "translator-credits", _("translator-credits"),
+                        NULL);
+
+  weather_service = gcal_context_get_weather_service (self->context);
+  adw_about_window_add_legal_section (ADW_ABOUT_WINDOW (about),
+                                      _("Weather"),
+                                      NULL,
+                                      GTK_LICENSE_CUSTOM,
+                                      gcal_weather_service_get_attribution (weather_service));
+
+  gtk_window_present (GTK_WINDOW (about));
 }
 
 static void
@@ -235,7 +205,7 @@ gcal_application_quit (GSimpleAction *simple,
 {
   GcalApplication *self = GCAL_APPLICATION (user_data);
 
-  gtk_widget_destroy (self->window);
+  gtk_window_destroy (GTK_WINDOW (self->window));
 }
 
 
@@ -307,7 +277,6 @@ gcal_application_activate (GApplication *application)
                                     "active-date", self->initial_date,
                                     NULL);
 
-      g_signal_connect (self->window, "destroy", G_CALLBACK (gtk_widget_destroyed), &self->window);
       g_object_add_weak_pointer (G_OBJECT (self->window), (gpointer*) &self->window);
       gtk_widget_show (self->window);
     }
@@ -359,9 +328,6 @@ gcal_application_startup (GApplication *app)
   if ((g_application_get_flags (app) & G_APPLICATION_IS_SERVICE) != 0)
     g_application_set_inactivity_timeout (app, 3 * 60 * 1000);
 
-  /*  initialize libhandy */
-  hdy_init();
-
   /* Startup the manager */
   gcal_context_startup (self->context);
 
@@ -413,7 +379,7 @@ gcal_application_command_line (GApplication            *app,
 
           initial_date = g_date_time_new (gcal_context_get_timezone (self->context),
                                           result.tm_year + 1900,
-                                          result.tm_mon,
+                                          result.tm_mon + 1,
                                           result.tm_mday,
                                           result.tm_hour,
                                           result.tm_min,
