@@ -28,6 +28,7 @@
 #include "gcal-debug.h"
 #include "gcal-event-popover.h"
 #include "gcal-event-widget.h"
+#include "gcal-overflow-bin.h"
 #include "gcal-utils.h"
 
 #define DESC_MAX_CHAR 200
@@ -50,10 +51,10 @@ struct _GcalEventWidget
   GDateTime          *dt_end;
 
   /* widgets */
-  GtkWidget          *color_box;
   GtkWidget          *horizontal_box;
-  GtkWidget          *hour_label;
-  GtkWidget          *stack;
+  GtkWidget          *main_widget;
+  GtkWidget          *timestamp_label;
+  GtkWidget          *squeezer;
   GtkWidget          *summary_label;
   GtkWidget          *vertical_box;
   GtkEventController *drag_source;
@@ -67,9 +68,7 @@ struct _GcalEventWidget
 
   GtkOrientation      orientation;
 
-  guint               vertical_label_source_id;
-  gboolean            vertical_value_to_set;
-  gboolean            vertical_labels;
+  GcalTimestampPolicy timestamp_policy;
 
   gint                old_width;
   gint                old_height;
@@ -84,6 +83,7 @@ enum
   PROP_DATE_END,
   PROP_DATE_START,
   PROP_EVENT,
+  PROP_TIMESTAMP_POLICY,
   PROP_ORIENTATION,
   NUM_PROPS
 };
@@ -111,59 +111,11 @@ G_DEFINE_TYPE_WITH_CODE (GcalEventWidget, gcal_event_widget, GTK_TYPE_WIDGET,
  */
 
 static gboolean
-can_hold_vertical_labels_with_height (GcalEventWidget *self,
-                                      gint             height)
+get_vertical_enabled (gpointer        user_data,
+                      GcalEvent      *event,
+                      GtkOrientation  orientation)
 {
-  gint total_height;
-
-  gtk_widget_measure (self->vertical_box,
-                      GTK_ORIENTATION_VERTICAL,
-                      -1,
-                      &total_height,
-                      NULL,
-                      NULL,
-                      NULL);
-
-  return height >= total_height;
-}
-
-static void
-set_vertical_labels (GcalEventWidget *self,
-                     gboolean         vertical)
-{
-  if (self->vertical_labels == vertical)
-    return;
-
-  gtk_stack_set_visible_child (GTK_STACK (self->stack), vertical ? self->vertical_box : self->horizontal_box);
-  self->vertical_labels = vertical;
-}
-
-static gboolean
-set_vertical_labels_in_idle_cb (gpointer data)
-{
-  GcalEventWidget *self = (GcalEventWidget*) data;
-
-  set_vertical_labels (self, self->vertical_value_to_set);
-
-  self->vertical_label_source_id = 0;
-  return G_SOURCE_REMOVE;
-}
-
-static void
-queue_set_vertical_labels (GcalEventWidget *self,
-                           gboolean         vertical)
-{
-  if (self->vertical_label_source_id > 0)
-    {
-      g_source_remove (self->vertical_label_source_id);
-      self->vertical_label_source_id = 0;
-    }
-
-  if (self->vertical_labels == vertical)
-    return;
-
-  self->vertical_value_to_set = vertical;
-  self->vertical_label_source_id = g_idle_add (set_vertical_labels_in_idle_cb, self);
+  return orientation == GTK_ORIENTATION_VERTICAL && !gcal_event_get_all_day (event);
 }
 
 static void
@@ -180,6 +132,7 @@ gcal_event_widget_update_style (GcalEventWidget *self)
   gtk_widget_remove_css_class (GTK_WIDGET (self), "slanted");
   gtk_widget_remove_css_class (GTK_WIDGET (self), "slanted-start");
   gtk_widget_remove_css_class (GTK_WIDGET (self), "slanted-end");
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "timed");
 
   /*
    * If the event's dates differs from the widget's dates,
@@ -217,22 +170,8 @@ gcal_event_widget_update_style (GcalEventWidget *self)
    */
   timed = !gcal_event_get_all_day (self->event) && !gcal_event_is_multiday (self->event);
 
-  gtk_widget_set_visible (self->color_box, timed);
-
   if (timed)
-    {
-      gtk_widget_add_css_class (GTK_WIDGET (self), "timed");
-
-      if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
-        {
-          gtk_widget_set_margin_start (self->stack, 0);
-          gtk_widget_set_margin_end (self->stack, 2);
-        }
-      else
-        {
-          gtk_widget_set_visible (self->color_box, FALSE);
-        }
-    }
+    gtk_widget_add_css_class (GTK_WIDGET (self), "timed");
 }
 
 static void
@@ -250,7 +189,10 @@ update_color (GcalEventWidget *self)
   date_compare = g_date_time_compare (self->dt_end, now);
 
   /* Fades out an event that's earlier than the current date */
-  gtk_widget_set_opacity (GTK_WIDGET (self), date_compare < 0 ? 0.6 : 1.0);
+  if (date_compare < 0)
+    gtk_widget_add_css_class (GTK_WIDGET (self), "dim-label");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "dim-label");
 
   /* Remove the old style class */
   if (self->css_class)
@@ -438,25 +380,37 @@ gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
   g_string_free (tooltip_mesg, TRUE);
 }
 
-static gchar*
-get_hour_label (GcalEventWidget *self)
+static void
+gcal_event_widget_update_timestamp (GcalEventWidget *self)
 {
-  g_autoptr (GDateTime) local_start_time;
+  g_autofree gchar *timestamp_str = NULL;
 
-  local_start_time = g_date_time_to_local (gcal_event_get_date_start (self->event));
+  if (GCAL_IS_EVENT (self->event) &&
+      self->timestamp_policy != GCAL_TIMESTAMP_POLICY_NONE)
+    {
+      g_autoptr (GDateTime) time = NULL;
 
-  if (self->clock_format_24h)
-    return g_date_time_format (local_start_time, "%R");
-  else
-    return g_date_time_format (local_start_time, "%I:%M %P");
+      if (self->timestamp_policy == GCAL_TIMESTAMP_POLICY_START)
+        time = g_date_time_to_local (gcal_event_get_date_start (self->event));
+      else
+        time = g_date_time_to_local (gcal_event_get_date_end (self->event));
+
+      if (gcal_event_get_all_day (self->event) || gcal_event_is_multiday (self->event))
+        timestamp_str = g_date_time_format (time, "%a %B %e");
+      else if (self->clock_format_24h)
+        timestamp_str = g_date_time_format (time, "%R");
+      else
+        timestamp_str = g_date_time_format (time, "%I:%M %P");
+    }
+
+  gtk_widget_set_visible (self->timestamp_label, timestamp_str != NULL);
+  gtk_label_set_label (GTK_LABEL (self->timestamp_label), timestamp_str);
 }
 
 static void
 gcal_event_widget_set_event_internal (GcalEventWidget *self,
                                       GcalEvent       *event)
 {
-  g_autofree gchar *hour_str = NULL;
-
   /*
    * This function is called only once, since the property is
    * set as CONSTRUCT_ONLY. Any other attempt to set an event
@@ -494,18 +448,15 @@ gcal_event_widget_set_event_internal (GcalEventWidget *self,
   /* Tooltip */
   gcal_event_widget_set_event_tooltip (self, event);
 
-  /* Hour label */
-  hour_str = get_hour_label (self);
-
-  gtk_widget_set_visible (self->hour_label, !gcal_event_get_all_day (event));
-  gtk_label_set_label (GTK_LABEL (self->hour_label), hour_str);
-
   /* Summary label */
   g_object_bind_property (event,
                           "summary",
                           self->summary_label,
                           "label",
                           G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+
+  gcal_event_widget_update_style (self);
+  gcal_event_widget_update_timestamp (self);
 }
 
 static void
@@ -585,53 +536,6 @@ on_event_popover_edit_cb (GtkWidget   *event_popover,
 }
 
 
-/*
- * GtkWidget overrides
- */
-
-static void
-gcal_event_widget_size_allocate (GtkWidget *widget,
-                                 gint       width,
-                                 gint       height,
-                                 gint       baseline)
-{
-  GcalEventWidget *self = GCAL_EVENT_WIDGET (widget);
-
-  gtk_widget_allocate (GTK_WIDGET (self->stack), width, height, baseline, NULL);
-
-  if (self->old_width != width || self->old_height != height)
-    {
-      if (self->orientation == GTK_ORIENTATION_HORIZONTAL || gcal_event_get_all_day (self->event))
-        return;
-
-      queue_set_vertical_labels (self, can_hold_vertical_labels_with_height (self, height));
-
-      self->old_width = width;
-      self->old_height = height;
-    }
-}
-
-static void
-gcal_event_widget_measure (GtkWidget      *widget,
-                           GtkOrientation  orientation,
-                           gint            for_size,
-                           gint           *minimum,
-                           gint           *natural,
-                           gint           *minimum_baseline,
-                           gint           *natural_baseline)
-{
-  GcalEventWidget *self = GCAL_EVENT_WIDGET (widget);
-
-  gtk_widget_measure (GTK_WIDGET (self->stack),
-                      orientation,
-                      for_size,
-                      minimum,
-                      natural,
-                      minimum_baseline,
-                      natural_baseline);
-}
-
-
 /*
  * GObject overrides
  */
@@ -669,10 +573,15 @@ gcal_event_widget_set_property (GObject      *object,
       gcal_event_widget_set_event_internal (self, g_value_get_object (value));
       break;
 
+    case PROP_TIMESTAMP_POLICY:
+      gcal_event_widget_set_timestamp_policy (self, g_value_get_enum (value));
+      break;
+
     case PROP_ORIENTATION:
       self->orientation = g_value_get_enum (value);
       gtk_widget_set_visible (self->vertical_box, self->orientation == GTK_ORIENTATION_VERTICAL);
       gcal_event_widget_update_style (self);
+      gcal_event_widget_update_timestamp (self);
       g_object_notify (object, "orientation");
       break;
 
@@ -707,6 +616,10 @@ gcal_event_widget_get_property (GObject      *object,
       g_value_set_object (value, self->event);
       break;
 
+    case PROP_TIMESTAMP_POLICY:
+      g_value_set_enum (value, self->timestamp_policy);
+      break;
+
     case PROP_ORIENTATION:
       g_value_set_enum (value, self->orientation);
       break;
@@ -721,8 +634,7 @@ gcal_event_widget_dispose (GObject *object)
 {
   GcalEventWidget *self = GCAL_EVENT_WIDGET (object);
 
-  g_clear_handle_id (&self->vertical_label_source_id, g_source_remove);
-  g_clear_pointer (&self->stack, gtk_widget_unparent);
+  g_clear_pointer (&self->main_widget, gtk_widget_unparent);
 
   G_OBJECT_CLASS (gcal_event_widget_parent_class)->dispose (object);
 }
@@ -757,9 +669,6 @@ gcal_event_widget_class_init (GcalEventWidgetClass *klass)
   object_class->dispose = gcal_event_widget_dispose;
   object_class->finalize = gcal_event_widget_finalize;
 
-  widget_class->measure = gcal_event_widget_measure;
-  widget_class->size_allocate = gcal_event_widget_size_allocate;
-
   /**
    * GcalEventWidget::context:
    *
@@ -817,6 +726,23 @@ gcal_event_widget_class_init (GcalEventWidgetClass *klass)
                                                         GCAL_TYPE_EVENT,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+  /**
+   * GcalEventWidget::timestamp-policy:
+   *
+   * The policy for this widget's timestamp.
+   *
+   * Whether to show the start time, end time, or no time for the
+   * event. Depending on the event's kind, it will be an hour or a day.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_TIMESTAMP_POLICY,
+                                   g_param_spec_enum ("timestamp-policy",
+                                                      "Timestamp Policy",
+                                                      "The policy for this widget's timestamp",
+                                                      GCAL_TYPE_TIMESTAMP_POLICY,
+                                                      GCAL_TIMESTAMP_POLICY_NONE,
+                                                      G_PARAM_READWRITE));
+
   /**
    * GcalEventWidget::orientation:
    *
@@ -835,20 +761,23 @@ gcal_event_widget_class_init (GcalEventWidgetClass *klass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/ui/gui/gcal-event-widget.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, color_box);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, drag_source);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, horizontal_box);
-  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, hour_label);
-  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, stack);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, main_widget);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, timestamp_label);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, squeezer);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, summary_label);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, vertical_box);
 
+  gtk_widget_class_bind_template_callback (widget_class, get_vertical_enabled);
   gtk_widget_class_bind_template_callback (widget_class, on_click_gesture_pressed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_click_gesture_release_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_drag_source_begin_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_drag_source_prepare_cb);
 
   gtk_widget_class_set_css_name (widget_class, "event");
+
+  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
 }
 
 static void
@@ -859,13 +788,15 @@ gcal_event_widget_init (GcalEventWidget *self)
   widget = GTK_WIDGET (self);
   self->clock_format_24h = is_clock_format_24h ();
 
+  g_type_ensure (GCAL_TYPE_OVERFLOW_BIN);
+
   gtk_widget_init_template (widget);
 
   gtk_widget_set_cursor_from_name (GTK_WIDGET (self), "pointer");
 
   /* Starts with horizontal */
   self->orientation = GTK_ORIENTATION_HORIZONTAL;
-  gtk_style_context_add_class (gtk_widget_get_style_context (widget), "horizontal");
+  gtk_widget_add_css_class (widget, "horizontal");
 }
 
 GtkWidget*
@@ -988,6 +919,31 @@ gcal_event_widget_set_date_start (GcalEventWidget *self,
     }
 }
 
+/**
+ * gcal_event_widget_set_timestamp_policy:
+ * @self: a #GcalEventWidget
+ * @policy: the timestamp policy
+ *
+ * Sets whether to show the start time, end time, or no time for the
+ * event. Depending on the event's kind, it will be an hour or a day.
+ */
+void
+gcal_event_widget_set_timestamp_policy (GcalEventWidget     *self,
+                                        GcalTimestampPolicy  policy)
+{
+  g_return_if_fail (GCAL_IS_EVENT_WIDGET (self));
+  g_return_if_fail (policy >= GCAL_TIMESTAMP_POLICY_NONE && policy <= GCAL_TIMESTAMP_POLICY_END);
+
+  if (self->timestamp_policy != policy)
+    {
+      self->timestamp_policy = policy;
+
+      gcal_event_widget_update_timestamp (self);
+
+      g_object_notify (G_OBJECT (self), "timestamp-policy");
+    }
+}
+
 /**
  * gcal_event_widget_show_preview:
  * @self: a #GcalEventWidget
