@@ -70,7 +70,7 @@ static GtkWidget*
 create_calendar_row (GcalManager  *manager,
                      GcalCalendar *calendar)
 {
-  cairo_surface_t *surface;
+  g_autoptr (GdkPaintable) paintable = NULL;
   const GdkRGBA *color;
   GtkWidget *row, *box, *icon, *label, *selected_icon;
   gboolean read_only;
@@ -85,22 +85,23 @@ create_calendar_row (GcalManager  *manager,
 
   /* The icon with the source color */
   color = gcal_calendar_get_color (calendar);
-  surface = get_circle_surface_from_color (color, 16);
-  icon = gtk_image_new_from_surface (surface);
+  paintable = get_circle_paintable_from_color (color, 16);
+  icon = gtk_image_new_from_paintable (paintable);
 
-  gtk_container_add (GTK_CONTAINER (box), icon);
+  gtk_box_append (GTK_BOX (box), icon);
 
-  gtk_style_context_add_class (gtk_widget_get_style_context (icon), "calendar-color-image");
+  gtk_widget_add_css_class (icon, "calendar-color-image");
 
   /* Source name label */
   label = gtk_label_new (gcal_calendar_get_name (calendar));
   gtk_widget_set_margin_end (label, 12);
 
-  gtk_container_add (GTK_CONTAINER (box), label);
+  gtk_box_append (GTK_BOX (box), label);
 
   /* Selected icon */
-  selected_icon = gtk_image_new_from_icon_name ("emblem-ok-symbolic", GTK_ICON_SIZE_BUTTON);
-  gtk_container_add (GTK_CONTAINER (box), selected_icon);
+  selected_icon = gtk_image_new_from_icon_name ("emblem-ok-symbolic");
+  gtk_widget_hide (selected_icon);
+  gtk_box_append (GTK_BOX (box), selected_icon);
 
   /* The row itself */
   row = gtk_list_box_row_new ();
@@ -108,7 +109,7 @@ create_calendar_row (GcalManager  *manager,
   read_only = gcal_calendar_is_read_only (calendar);
   gtk_widget_set_sensitive (row, !read_only);
 
-  gtk_container_add (GTK_CONTAINER (row), box);
+  gtk_list_box_row_set_child (GTK_LIST_BOX_ROW (row), box);
 
   /* Setup also a cool tooltip */
   get_source_parent_name_color (manager, gcal_calendar_get_source (calendar), &parent_name, NULL);
@@ -130,12 +131,6 @@ create_calendar_row (GcalManager  *manager,
   g_object_set_data (G_OBJECT (row), "color-icon", icon);
   g_object_set_data (G_OBJECT (row), "name-label", label);
 
-  gtk_widget_show (label);
-  gtk_widget_show (icon);
-  gtk_widget_show (box);
-  gtk_widget_show (row);
-
-  g_clear_pointer (&surface, cairo_surface_destroy);
   g_free (parent_name);
   g_free (tooltip);
 
@@ -150,33 +145,26 @@ static GtkWidget*
 get_row_for_calendar (GcalQuickAddPopover *self,
                       GcalCalendar        *calendar)
 {
-  GList *children, *l;
   GtkWidget *row;
 
-  row = NULL;
-  children = gtk_container_get_children (GTK_CONTAINER (self->calendars_listbox));
-
-  for (l = children; l != NULL; l = g_list_next (l))
+  for (row = gtk_widget_get_first_child (self->calendars_listbox);
+       row;
+       row = gtk_widget_get_next_sibling (row))
     {
-      GcalCalendar *row_calendar = g_object_get_data (l->data, "calendar");
+      GcalCalendar *row_calendar = g_object_get_data (G_OBJECT (row), "calendar");
 
       if (row_calendar == calendar)
-        {
-          row = l->data;
-          break;
-        }
+        return row;
     }
 
-  g_list_free (children);
-
-  return row;
+  return NULL;
 }
 
 static void
 select_row (GcalQuickAddPopover *self,
             GtkListBoxRow       *row)
 {
-  cairo_surface_t *surface;
+  g_autoptr (GdkPaintable) paintable = NULL;
   const GdkRGBA *color;
   GcalCalendar *calendar;
   GtkWidget *icon;
@@ -200,8 +188,8 @@ select_row (GcalQuickAddPopover *self,
   gtk_label_set_label (GTK_LABEL (self->calendar_name_label), gcal_calendar_get_name (calendar));
 
   color = gcal_calendar_get_color (calendar);
-  surface = get_circle_surface_from_color (color, 16);
-  gtk_image_set_from_surface (GTK_IMAGE (self->color_image), surface);
+  paintable = get_circle_paintable_from_color (color, 16);
+  gtk_image_set_from_paintable (GTK_IMAGE (self->color_image), paintable);
 
   /* Return to the events page */
   gtk_stack_set_visible_child_name (GTK_STACK (self->stack), "events");
@@ -209,20 +197,21 @@ select_row (GcalQuickAddPopover *self,
   /* Focus back the event Entry */
   if (gtk_widget_get_visible (GTK_WIDGET (self)))
     gtk_entry_grab_focus_without_selecting (GTK_ENTRY (self->summary_entry));
-
-  g_clear_pointer (&surface, cairo_surface_destroy);
 }
 
 static gint
 get_number_of_days_from_today (GDateTime *day)
 {
+  g_autoptr (GDateTime) now = NULL;
   GDate today, event_day;
   gint n_days;
 
+  now = g_date_time_new_now_local ();
+
   g_date_set_dmy (&today,
-                  g_date_time_get_day_of_month (g_date_time_new_now_local ()),
-                  g_date_time_get_month (g_date_time_new_now_local ()),
-                  g_date_time_get_year (g_date_time_new_now_local ()));
+                  g_date_time_get_day_of_month (now),
+                  g_date_time_get_month (now),
+                  g_date_time_get_year (now));
 
   g_date_set_dmy (&event_day,
                   g_date_time_get_day_of_month (day),
@@ -240,7 +229,7 @@ get_date_string_for_multiday (GDateTime *start,
   gint n_days;
   gchar *start_date_str, *end_date_str, *date_string;
 
-  gchar *start_date_weekdays_strings [] = {
+  const gchar *start_date_weekdays_strings[] = {
     N_("from next Monday"),
     N_("from next Tuesday"),
     N_("from next Wednesday"),
@@ -251,7 +240,7 @@ get_date_string_for_multiday (GDateTime *start,
     NULL
   };
 
-  gchar *end_date_weekdays_strings [] = {
+  const gchar *end_date_weekdays_strings[] = {
     N_("to next Monday"),
     N_("to next Tuesday"),
     N_("to next Wednesday"),
@@ -262,7 +251,7 @@ get_date_string_for_multiday (GDateTime *start,
     NULL
   };
 
-  gchar *month_names [] = {
+  const gchar *month_names[] = {
     N_("January"),
     N_("February"),
     N_("March"),
@@ -295,9 +284,9 @@ get_date_string_for_multiday (GDateTime *start,
     }
   else if (n_days < -1 && n_days > -8)
     {
-      gchar *start_weekday_str;
+      const gchar *start_weekday_str;
 
-      start_weekday_str = start_date_weekdays_strings [g_date_time_get_day_of_week (start) - 1];
+      start_weekday_str = gettext (start_date_weekdays_strings[g_date_time_get_day_of_week (start) - 1]);
       start_date_str = g_strdup_printf ("%s", start_weekday_str);
     }
   else
@@ -310,7 +299,7 @@ get_date_string_for_multiday (GDateTime *start,
        * name and a date of month.
        */
       start_date_str = g_strdup_printf (_("from %1$s %2$s"),
-                                        month_names [g_date_time_get_month (start) - 1],
+                                        gettext (month_names[g_date_time_get_month (start) - 1]),
                                         day_number_str);
       g_free (day_number_str);
     }
@@ -332,9 +321,9 @@ get_date_string_for_multiday (GDateTime *start,
     }
   else if (n_days < -1 && n_days > -8)
     {
-      gchar *end_weekday_str;
+      const gchar *end_weekday_str;
 
-      end_weekday_str = end_date_weekdays_strings [g_date_time_get_day_of_week (end) - 1];
+      end_weekday_str = gettext (end_date_weekdays_strings[g_date_time_get_day_of_week (end) - 1]);
       end_date_str = g_strdup_printf ("%s", end_weekday_str);
     }
   else
@@ -347,7 +336,7 @@ get_date_string_for_multiday (GDateTime *start,
        * name and a date of month.
        */
       end_date_str = g_strdup_printf (_("to %1$s %2$s"),
-                                      month_names [g_date_time_get_month (end) - 1],
+                                      gettext (month_names[g_date_time_get_month (end) - 1]),
                                       day_number_str);
       g_free (day_number_str);
     }
@@ -383,7 +372,7 @@ get_date_string_for_day (GDateTime *day)
   else if (n_days < -1 && n_days > -8)
     {
        gchar *event_weekday;
-       gchar *event_weekday_names [] = {
+       const gchar *event_weekday_names[] = {
          N_("New Event next Monday"),
          N_("New Event next Tuesday"),
          N_("New Event next Wednesday"),
@@ -400,7 +389,7 @@ get_date_string_for_day (GDateTime *day)
   else
     {
       const gchar *event_month;
-      gchar *event_month_names [] = {
+      const gchar *event_month_names[] = {
         /* Translators: %d is the numeric day of month */
         N_("New Event on January %d"),
         N_("New Event on February %d"),
@@ -521,9 +510,9 @@ on_calendar_added (GcalManager         *manager,
   default_calendar = gcal_manager_get_default_calendar (manager);
   row = create_calendar_row (manager, calendar);
 
-  gtk_container_add (GTK_CONTAINER (self->calendars_listbox), row);
+  gtk_list_box_append (GTK_LIST_BOX (self->calendars_listbox), row);
 
-  /* Select the default source whe first adding events */
+  /* Select the default source when first adding events */
   if (calendar == default_calendar && !self->selected_row)
     select_row (self, GTK_LIST_BOX_ROW (row));
 }
@@ -533,7 +522,7 @@ on_calendar_changed (GcalManager         *manager,
                      GcalCalendar        *calendar,
                      GcalQuickAddPopover *self)
 {
-  cairo_surface_t *surface;
+  g_autoptr (GdkPaintable) paintable = NULL;
   const GdkRGBA *color;
   GtkWidget *row, *color_icon, *name_label;
   gboolean read_only;
@@ -545,7 +534,7 @@ on_calendar_changed (GcalManager         *manager,
   if (read_only)
     {
       if (row)
-        gtk_container_remove (GTK_CONTAINER (self->calendars_listbox), row);
+        gtk_list_box_remove (GTK_LIST_BOX (self->calendars_listbox), row);
 
       return;
     }
@@ -564,10 +553,8 @@ on_calendar_changed (GcalManager         *manager,
 
   /* Setup the source color, in case it changed */
   color = gcal_calendar_get_color (calendar);
-  surface = get_circle_surface_from_color (color, 16);
-  gtk_image_set_from_surface (GTK_IMAGE (color_icon), surface);
-
-  g_clear_pointer (&surface, cairo_surface_destroy);
+  paintable = get_circle_paintable_from_color (color, 16);
+  gtk_image_set_from_paintable (GTK_IMAGE (color_icon), paintable);
 
   /* Also setup the row name, in case we just changed the source name */
   gtk_label_set_text (GTK_LABEL (name_label), gcal_calendar_get_name (calendar));
@@ -594,7 +581,7 @@ on_calendar_removed (GcalManager         *manager,
   if (!row)
     return;
 
-  gtk_container_remove (GTK_CONTAINER (self->calendars_listbox), row);
+  gtk_list_box_remove (GTK_LIST_BOX (self->calendars_listbox), row);
 }
 
 /* Sort the calendars by their display name */
@@ -684,7 +671,7 @@ edit_or_create_event (GcalQuickAddPopover *self,
 
   /* Gather the summary */
   if (gtk_entry_get_text_length (GTK_ENTRY (self->summary_entry)) > 0)
-    summary = gtk_entry_get_text (GTK_ENTRY (self->summary_entry));
+    summary = gtk_editable_get_text (GTK_EDITABLE (self->summary_entry));
   else
     summary = _("Unnamed event");
 
@@ -827,7 +814,7 @@ gcal_quick_add_popover_closed (GtkPopover *popover)
   self = GCAL_QUICK_ADD_POPOVER (popover);
 
   /* Clear text */
-  gtk_entry_set_text (GTK_ENTRY (self->summary_entry), "");
+  gtk_editable_set_text (GTK_EDITABLE (self->summary_entry), "");
 
   /* Select the default row again */
   update_default_calendar_row (self);
@@ -854,7 +841,7 @@ gcal_quick_add_popover_class_init (GcalQuickAddPopoverClass *klass)
    * @self: a #GcalQuickAddPopover
    * @event: the new #GcalEvent
    *
-   * Emited when the user clicks 'Edit event' button.
+   * Emitted when the user clicks 'Edit event' button.
    */
   signals[EDIT_EVENT] = g_signal_new ("edit-event",
                                       GCAL_TYPE_QUICK_ADD_POPOVER,
