@@ -694,7 +694,9 @@ on_event_editor_dialog_remove_event_cb (GcalEventEditorDialog *edit_dialog,
   GCAL_ENTRY;
 
   has_deleted_event = self->delete_event_toast != NULL;
-  g_clear_pointer (&self->delete_event_toast, adw_toast_dismiss);
+  if (self->delete_event_toast)
+    adw_toast_dismiss (self->delete_event_toast);
+  self->delete_event_toast = NULL;
 
   toast = adw_toast_new (has_deleted_event ? _("Another event deleted") : _("Event deleted"));
   adw_toast_set_timeout (toast, 5);
@@ -704,7 +706,7 @@ on_event_editor_dialog_remove_event_cb (GcalEventEditorDialog *edit_dialog,
   g_object_set_data (G_OBJECT (toast), "modifier", GINT_TO_POINTER (modifier));
   g_signal_connect (toast, "dismissed", G_CALLBACK (on_toast_dismissed_cb), self);
 
-  adw_toast_overlay_add_toast (self->overlay, toast);
+  adw_toast_overlay_add_toast (self->overlay, g_object_ref (toast));
   self->delete_event_toast = g_steal_pointer (&toast);
 
   /* hide widget of the event */
