@@ -29,7 +29,7 @@
 
 struct _GcalMonthCell
 {
-  GtkEventBox         parent;
+  AdwBin              parent;
 
   GDateTime          *date;
   guint               n_overflow;
@@ -46,16 +46,12 @@ struct _GcalMonthCell
   gboolean            different_month;
   gboolean            selected;
 
-  /* internal fields */
-  gboolean            hovered : 1;
-  gboolean            pressed : 1;
-
   GcalContext        *context;
 
   GcalWeatherInfo    *weather_info;
 };
 
-G_DEFINE_TYPE (GcalMonthCell, gcal_month_cell, GTK_TYPE_EVENT_BOX)
+G_DEFINE_TYPE (GcalMonthCell, gcal_month_cell, ADW_TYPE_BIN)
 
 enum
 {
@@ -82,161 +78,33 @@ static void
 update_style_flags (GcalMonthCell *self)
 {
   g_autoptr (GDateTime) today = NULL;
-  GtkStyleContext *context;
-  GtkStateFlags flags;
-  GtkWidget *widget;
-
-  widget = GTK_WIDGET (self);
-  flags = gtk_widget_get_state_flags (widget);
-
-  if (self->hovered)
-    flags |= GTK_STATE_FLAG_PRELIGHT;
-  else
-    flags &= ~GTK_STATE_FLAG_PRELIGHT;
-
-  gtk_widget_set_state_flags (widget, flags, TRUE);
 
   /* Today */
   today = g_date_time_new_now_local ();
-  context = gtk_widget_get_style_context (GTK_WIDGET (self));
 
   if (gcal_date_time_compare_date (self->date, today) == 0)
-    gtk_style_context_add_class (context, "today");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "today");
   else
-    gtk_style_context_remove_class (context, "today");
-}
-
-static gboolean
-should_propagate_button_event (GtkWidget      *widget,
-                               GdkEventButton *event_button)
-{
-  GcalMonthCell *self;
-  gdouble x, y;
-  gint button_height, height;
-
-  self = GCAL_MONTH_CELL (widget);
-  height = gtk_widget_get_allocated_height (widget);
-  button_height = gtk_widget_get_allocated_height (self->overflow_button);
-
-  x = event_button->x;
-  y = event_button->y;
-
-  if (!gcal_translate_child_window_position (widget, event_button->window, x, y, &x, &y))
-    return GDK_EVENT_PROPAGATE;
-
-  if (self->n_overflow > 0 && y > height - button_height)
-    return GDK_EVENT_STOP;
-
-  return GDK_EVENT_PROPAGATE;
-}
-
-
-/*
- * Callbacks
- */
-
-static void
-day_changed_cb (GcalClock     *clock,
-                GcalMonthCell *self)
-{
-  update_style_flags (self);
-}
-
-static gboolean
-enter_notify_event_cb (GtkWidget        *widget,
-                       GdkEventCrossing *event_crossing,
-                       GcalMonthCell    *self)
-{
-  self->hovered = TRUE;
-
-  update_style_flags (self);
-
-  return GDK_EVENT_PROPAGATE;
-}
-
-static gboolean
-leave_notify_event_cb (GtkWidget        *widget,
-                       GdkEventCrossing *event_crossing,
-                       GcalMonthCell    *self)
-{
-  self->hovered = FALSE;
-
-  update_style_flags (self);
-
-  return GDK_EVENT_PROPAGATE;
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "today");
 }
 
 static void
-overflow_button_clicked_cb (GtkWidget     *button,
-                            GcalMonthCell *self)
+move_event (GcalMonthCell         *self,
+            GcalEvent             *event,
+            GcalRecurrenceModType  mod_type)
 {
-  g_signal_emit (self, signals[SHOW_OVERFLOW], 0, button);
-}
 
-/*
- * GtkWidget overrides
- */
-static gboolean
-gcal_month_cell_drag_motion (GtkWidget      *widget,
-                             GdkDragContext *context,
-                             gint            x,
-                             gint            y,
-                             guint           time)
-{
-  GcalMonthCell *self;
-
-  self = GCAL_MONTH_CELL (widget);
-
-  if (self->different_month)
-    gtk_drag_unhighlight (widget);
-  else
-    gtk_drag_highlight (widget);
-
-  gdk_drag_status (context, self->different_month ? 0 : GDK_ACTION_MOVE, time);
-
-  return !self->different_month;
-}
-
-static gboolean
-gcal_month_cell_drag_drop (GtkWidget      *widget,
-                           GdkDragContext *context,
-                           gint            x,
-                           gint            y,
-                           guint           time)
-{
   g_autoptr (GcalEvent) changed_event = NULL;
-  GcalRecurrenceModType mod;
-  GcalMonthCell *self;
-  GcalCalendar *calendar;
-  GtkWidget *event_widget;
-  GDateTime *start_dt, *end_dt;
-  GcalEvent *event;
+  g_autoptr (GDateTime) start_dt = NULL;
+  GTimeSpan timespan = 0;
+  GDateTime *end_dt;
   gint diff;
   gint start_month, current_month;
   gint start_year, current_year;
-  GTimeSpan timespan = 0;
-
-  self = GCAL_MONTH_CELL (widget);
-  event_widget = gtk_drag_get_source_widget (context);
-  mod = GCAL_RECURRENCE_MOD_THIS_ONLY;
 
   GCAL_ENTRY;
 
-  if (!GCAL_IS_EVENT_WIDGET (event_widget))
-    GCAL_RETURN (FALSE);
-
-  if (self->different_month)
-    GCAL_RETURN (FALSE);
-
-  event = gcal_event_widget_get_event (GCAL_EVENT_WIDGET (event_widget));
   changed_event = gcal_event_new_from_event (event);
-  calendar = gcal_event_get_calendar (changed_event);
-
-  if (gcal_event_has_recurrence (changed_event) &&
-      !ask_recurrence_modification_type (widget, &mod, calendar))
-    {
-      GCAL_GOTO (out);
-    }
 
   /* Move the event's date */
   start_dt = gcal_event_get_date_start (changed_event);
@@ -257,89 +125,106 @@ gcal_month_cell_drag_drop (GtkWidget      *widget,
 
   diff = g_date_time_get_day_of_month (self->date) - g_date_time_get_day_of_month (start_dt);
 
-  if (diff != 0 ||
-      current_month != start_month ||
-      current_year != start_year)
+  if (diff != 0 || current_month != start_month || current_year != start_year)
     {
-      g_autoptr (GDateTime) new_start = NULL;
-
-       new_start = g_date_time_add_days (start_dt, diff);
+      g_autoptr (GDateTime) new_start = g_date_time_add_days (start_dt, diff);
 
       gcal_event_set_date_start (changed_event, new_start);
 
       /* The event may have a NULL end date, so we have to check it here */
       if (end_dt)
         {
-          GDateTime *new_end = g_date_time_add (new_start, timespan);
+          g_autoptr (GDateTime) new_end = g_date_time_add (new_start, timespan);
 
           gcal_event_set_date_end (changed_event, new_end);
-          g_clear_pointer (&new_end, g_date_time_unref);
         }
 
-      gcal_manager_update_event (gcal_context_get_manager (self->context), changed_event, mod);
+      gcal_manager_update_event (gcal_context_get_manager (self->context), changed_event, mod_type);
     }
+}
 
-  g_clear_pointer (&start_dt, g_date_time_unref);
 
-out:
-  /* Cancel the DnD */
-  gtk_drag_unhighlight (widget);
-  gtk_drag_finish (context, TRUE, FALSE, time);
+/*
+ * Callbacks
+ */
 
-  GCAL_RETURN (TRUE);
+static void
+day_changed_cb (GcalClock     *clock,
+                GcalMonthCell *self)
+{
+  update_style_flags (self);
 }
 
 static void
-gcal_month_cell_drag_leave (GtkWidget      *widget,
-                            GdkDragContext *context,
-                            guint           time)
+overflow_button_clicked_cb (GtkWidget     *button,
+                            GcalMonthCell *self)
 {
-  gtk_drag_unhighlight (widget);
+  g_signal_emit (self, signals[SHOW_OVERFLOW], 0, button);
 }
 
 static gboolean
-gcal_month_cell_button_press_event (GtkWidget      *widget,
-                                    GdkEventButton *event_button)
+on_drop_target_accept_cb (GtkDropTarget *drop_target,
+                          GdkDrop       *drop,
+                          GcalMonthCell *self)
 {
-  GcalMonthCell *self = GCAL_MONTH_CELL (widget);
+  GCAL_ENTRY;
 
-  self->pressed = TRUE;
+  if ((gdk_drop_get_actions (drop) & gtk_drop_target_get_actions (drop_target)) == 0)
+    GCAL_RETURN (FALSE);
 
-  return should_propagate_button_event (widget, event_button);
+  if (!gdk_content_formats_contain_gtype (gdk_drop_get_formats (drop), GCAL_TYPE_EVENT_WIDGET))
+    GCAL_RETURN (FALSE);
+
+  GCAL_RETURN (!self->different_month);
+}
+
+static void
+on_ask_recurrence_response_cb (GcalEvent             *event,
+                               GcalRecurrenceModType  mod_type,
+                               gpointer               user_data)
+{
+  GcalMonthCell *self = GCAL_MONTH_CELL (user_data);
+
+  if (mod_type != GCAL_RECURRENCE_MOD_NONE)
+    move_event (self, event, mod_type);
 }
 
 static gboolean
-gcal_month_cell_button_release_event (GtkWidget      *widget,
-                                      GdkEventButton *event_button)
+on_drop_target_drop_cb (GtkDropTarget *drop_target,
+                        const GValue  *value,
+                        gdouble        x,
+                        gdouble        y,
+                        GcalMonthCell *self)
 {
-  GcalMonthCell *self = GCAL_MONTH_CELL (widget);
+  GcalEventWidget *event_widget;
+  GcalEvent *event;
 
-  if (!self->pressed)
-    return GDK_EVENT_STOP;
+  GCAL_ENTRY;
 
-  self->pressed = FALSE;
+  if (self->different_month)
+    GCAL_RETURN (FALSE);
 
-  return should_propagate_button_event (widget, event_button);
-}
+  if (!G_VALUE_HOLDS (value, GCAL_TYPE_EVENT_WIDGET))
+    GCAL_RETURN (FALSE);
 
-static gboolean
-gcal_month_cell_draw (GtkWidget *widget,
-                      cairo_t   *cr)
-{
-  GtkStyleContext *context;
-  gint width, height;
+  event_widget = g_value_get_object (value);
 
-  width = gtk_widget_get_allocated_width (widget);
-  height = gtk_widget_get_allocated_height (widget);
-  context = gtk_widget_get_style_context (widget);
+  event = gcal_event_widget_get_event (event_widget);
 
-  gtk_render_background (context, cr, 0, 0, width, height);
-  gtk_render_frame (context, cr, 0, 0, width, height);
+  if (gcal_event_has_recurrence (event))
+    {
+      gcal_utils_ask_recurrence_modification_type (GTK_WIDGET (self),
+                                                   event,
+                                                   FALSE,
+                                                   on_ask_recurrence_response_cb,
+                                                   self);
+    }
+  else
+    {
+      move_event (self, event, GCAL_RECURRENCE_MOD_THIS_ONLY);
+    }
 
-  if (gtk_widget_has_focus (widget))
-    gtk_render_focus (context, cr, 0, 0, width, height);
-
-  return GTK_WIDGET_CLASS (gcal_month_cell_parent_class)->draw (widget, cr);
+  GCAL_RETURN (TRUE);
 }
 
 
@@ -409,13 +294,6 @@ gcal_month_cell_class_init (GcalMonthCellClass *klass)
   object_class->set_property = gcal_month_cell_set_property;
   object_class->get_property = gcal_month_cell_get_property;
 
-  widget_class->button_press_event = gcal_month_cell_button_press_event;
-  widget_class->button_release_event = gcal_month_cell_button_release_event;
-  widget_class->drag_motion = gcal_month_cell_drag_motion;
-  widget_class->drag_drop = gcal_month_cell_drag_drop;
-  widget_class->drag_leave = gcal_month_cell_drag_leave;
-  widget_class->draw = gcal_month_cell_draw;
-
   signals[SHOW_OVERFLOW] = g_signal_new ("show-overflow",
                                          GCAL_TYPE_MONTH_CELL,
                                          G_SIGNAL_RUN_LAST,
@@ -442,8 +320,6 @@ gcal_month_cell_class_init (GcalMonthCellClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, temp_label);
   gtk_widget_class_bind_template_child (widget_class, GcalMonthCell, weather_icon);
 
-  gtk_widget_class_bind_template_callback (widget_class, enter_notify_event_cb);
-  gtk_widget_class_bind_template_callback (widget_class, leave_notify_event_cb);
   gtk_widget_class_bind_template_callback (widget_class, overflow_button_clicked_cb);
 
   gtk_widget_class_set_css_name (widget_class, "monthcell");
@@ -452,14 +328,14 @@ gcal_month_cell_class_init (GcalMonthCellClass *klass)
 static void
 gcal_month_cell_init (GcalMonthCell *self)
 {
+  GtkDropTarget *drop_target;
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
-  /* Setup the month cell as a drag n' drop destination */
-  gtk_drag_dest_set (GTK_WIDGET (self),
-                     0,
-                     NULL,
-                     0,
-                     GDK_ACTION_MOVE);
+  drop_target = gtk_drop_target_new (GCAL_TYPE_EVENT_WIDGET, GDK_ACTION_COPY);
+  g_signal_connect (drop_target, "accept", G_CALLBACK (on_drop_target_accept_cb), self);
+  g_signal_connect (drop_target, "drop", G_CALLBACK (on_drop_target_drop_cb), self);
+  gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (drop_target));
 }
 
 GtkWidget*
@@ -533,7 +409,7 @@ gcal_month_cell_set_weather (GcalMonthCell   *self,
       icon_name = gcal_weather_info_get_icon_name (info);
       temp_str = gcal_weather_info_get_temperature (info);
 
-      gtk_image_set_from_icon_name (self->weather_icon, icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
+      gtk_image_set_from_icon_name (self->weather_icon, icon_name);
       gtk_label_set_text (self->temp_label, temp_str);
     }
 }
@@ -550,8 +426,6 @@ void
 gcal_month_cell_set_different_month (GcalMonthCell *self,
                                      gboolean       different)
 {
-  GtkStyleContext *context;
-
   g_return_if_fail (GCAL_IS_MONTH_CELL (self));
 
   if (self->different_month == different)
@@ -559,12 +433,10 @@ gcal_month_cell_set_different_month (GcalMonthCell *self,
 
   self->different_month = different;
 
-  context = gtk_widget_get_style_context (GTK_WIDGET (self));
-
   if (different)
-    gtk_style_context_add_class (context, "out-of-month");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "out-of-month");
   else
-    gtk_style_context_remove_class (context, "out-of-month");
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "out-of-month");
 }
 
 GcalContext*
@@ -631,8 +503,8 @@ gcal_month_cell_get_content_space (GcalMonthCell *self)
   g_return_val_if_fail (GCAL_IS_MONTH_CELL (self), -1);
 
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
-  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
-  gtk_style_context_get_border (context, gtk_style_context_get_state (context), &border);
+  gtk_style_context_get_padding (context, &padding);
+  gtk_style_context_get_border (context, &border);
 
   return gtk_widget_get_allocated_height (GTK_WIDGET (self)) -
          gcal_month_cell_get_header_height (self) -
@@ -650,8 +522,8 @@ gcal_month_cell_get_header_height (GcalMonthCell *self)
   g_return_val_if_fail (GCAL_IS_MONTH_CELL (self), -1);
 
   context = gtk_widget_get_style_context (GTK_WIDGET (self->header_box));
-  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
-  gtk_style_context_get_border (context, gtk_style_context_get_state (context), &border);
+  gtk_style_context_get_padding (context, &padding);
+  gtk_style_context_get_border (context, &border);
 
   return gtk_widget_get_allocated_height (self->header_box) +
          gtk_widget_get_margin_top (self->header_box) +
