@@ -442,6 +442,7 @@ on_drop_target_drop_cb (GtkDropTarget *drop_target,
 
       gcal_utils_ask_recurrence_modification_type (GTK_WIDGET (self),
                                                    event,
+                                                   FALSE,
                                                    on_ask_recurrence_response_cb,
                                                    data);
     }
@@ -794,7 +795,7 @@ gcal_week_grid_size_allocate (GtkWidget *widget,
           if (ltr)
             x = column_width * i + offset + 1;
           else
-            x = width - event_width - (column_width * i + offset + x + 1);
+            x = width - event_width - (column_width * i + offset + 1);
 
           /* Setup the child position and size */
           child_allocation.x = x;
