@@ -35,14 +35,15 @@ gboolean
 gcal_set_date_time (GDateTime **dest,
                     GDateTime  *src)
 {
-  gboolean changed = *dest != src;
+  if (*dest == src)
+    return FALSE;
 
   gcal_clear_date_time (dest);
 
   if (src)
     *dest = g_date_time_ref (src);
 
-  return changed;
+  return TRUE;
 }
 
 /**
@@ -218,10 +219,12 @@ gcal_date_time_from_icaltime (const ICalTime *date)
 {
   g_autoptr (GTimeZone) tz = NULL;
   g_autoptr (GDateTime) dt = NULL;
+  const gchar *identifier;
   ICalTimezone *zone;
 
   zone = i_cal_time_get_timezone (date);
-  tz = (zone && i_cal_timezone_get_location (zone)) ? g_time_zone_new (i_cal_timezone_get_location (zone)) : g_time_zone_new_utc ();
+  identifier = zone ? i_cal_timezone_get_location (zone) : NULL;
+  tz = identifier ? g_time_zone_new_identifier (identifier) : g_time_zone_new_utc ();
   dt = g_date_time_new (tz,
                         i_cal_time_get_year (date),
                         i_cal_time_get_month (date),
