@@ -132,39 +132,27 @@ gcal_get_month_name (gint i)
 }
 
 /**
- * gcal_get_surface_from_color:
+ * gcal_get_paintable_from_color:
  * @color: a #GdkRGBA
  * @size: the size of the surface
  *
  * Creates a squared surface filled with @color. The
  * surface is always @size x @size.
  *
- * Returns: (transfer full): a #cairo_surface_t
+ * Returns: (transfer full): a #GdkPaintable
  */
-cairo_surface_t*
-gcal_get_surface_from_color (const GdkRGBA *color,
-                             gint           size)
+GdkPaintable*
+gcal_get_paintable_from_color (const GdkRGBA *color,
+                               gint           size)
 {
-  cairo_surface_t *surface;
-  cairo_t *cr;
+  g_autoptr (GtkSnapshot) snapshot = gtk_snapshot_new ();
 
-  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, size, size);
-  cr = cairo_create (surface);
-
-  cairo_set_source_rgba (cr,
-                         color->red,
-                         color->green,
-                         color->blue,
-                         color->alpha);
-  cairo_rectangle (cr, 0, 0, size, size);
-  cairo_fill (cr);
-  cairo_destroy (cr);
-
-  return surface;
+  gtk_snapshot_append_color (snapshot, color, &GRAPHENE_RECT_INIT (0, 0, size, size));
+  return gtk_snapshot_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (size, size));
 }
 
 /**
- * get_circle_surface_from_color:
+ * get_circle_paintable_from_color:
  * @color: a #GdkRGBA
  * @size: the size of the surface
  *
@@ -173,26 +161,68 @@ gcal_get_surface_from_color (const GdkRGBA *color,
  *
  * Returns: (transfer full): a #cairo_surface_t
  */
-cairo_surface_t*
-get_circle_surface_from_color (const GdkRGBA *color,
-                               gint           size)
+GdkPaintable*
+get_circle_paintable_from_color (const GdkRGBA *color,
+                                 gint           size)
 {
-  cairo_surface_t *surface;
-  cairo_t *cr;
+  g_autoptr (GtkSnapshot) snapshot = NULL;
+  GskRoundedRect rect;
 
-  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, size, size);
-  cr = cairo_create (surface);
+  snapshot = gtk_snapshot_new ();
 
-  cairo_set_source_rgba (cr,
-                         color->red,
-                         color->green,
-                         color->blue,
-                         color->alpha);
-  cairo_arc (cr, size / 2.0, size / 2.0, size / 2.0, 0., 2 * M_PI);
-  cairo_fill (cr);
-  cairo_destroy (cr);
+  gtk_snapshot_push_rounded_clip (snapshot,
+                                  gsk_rounded_rect_init_from_rect (&rect,
+                                                                   &GRAPHENE_RECT_INIT (0, 0, size, size),
+                                                                   size / 2.0));
 
-  return surface;
+  gtk_snapshot_append_color (snapshot, color, &GRAPHENE_RECT_INIT (0, 0, size, size));
+
+  gtk_snapshot_pop (snapshot);
+
+  return gtk_snapshot_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (size, size));
+}
+
+/**
+ * paintable_to_pixbuf:
+ * @color: a #GdkRGBA
+ * @size: the size of the surface
+ *
+ * Creates a circular surface filled with @color. The
+ * surface is always @size x @size.
+ *
+ * Returns: (transfer full): a #cairo_surface_t
+ */
+GdkPixbuf*
+paintable_to_pixbuf (GdkPaintable *paintable)
+{
+  g_autoptr (GtkSnapshot) snapshot = NULL;
+  g_autoptr (GskRenderNode) node = NULL;
+  g_autoptr (GskRenderer) renderer = NULL;
+  g_autoptr (GdkTexture) texture = NULL;
+  g_autoptr (GError) error = NULL;
+  graphene_rect_t viewport;
+
+  snapshot = gtk_snapshot_new ();
+  gdk_paintable_snapshot (paintable, snapshot,
+                          gdk_paintable_get_intrinsic_width (paintable),
+                          gdk_paintable_get_intrinsic_height (paintable));
+  node = gtk_snapshot_free_to_node (g_steal_pointer (&snapshot));
+
+  renderer = gsk_cairo_renderer_new ();
+  gsk_renderer_realize (renderer, NULL, &error);
+  if (error)
+    {
+      g_warning ("Couldn't realize Cairo renderer: %s", error->message);
+      return NULL;
+    }
+
+  viewport = GRAPHENE_RECT_INIT (0, 0,
+                                 gdk_paintable_get_intrinsic_width (paintable),
+                                 gdk_paintable_get_intrinsic_height (paintable));
+  texture = gsk_renderer_render_texture (renderer, node, &viewport);
+  gsk_renderer_unrealize (renderer);
+
+  return gdk_pixbuf_get_from_texture (texture);
 }
 
 /**
@@ -680,64 +710,54 @@ fix_popover_menu_icons (GtkPopover *popover)
   GtkWidget *popover_stack;
   GtkWidget *menu_section;
   GtkWidget *menu_section_box;
-  GList *stack_children;
-  GList *menu_section_children;
-  GList *menu_section_box_children, *aux;
+  GtkWidget *widget;
+  GtkWidget *child;
 
-  popover_stack = gtk_bin_get_child (GTK_BIN (popover));
-  stack_children = gtk_container_get_children (GTK_CONTAINER (popover_stack));
+  /* GtkScrolledWindow */
+  widget = gtk_popover_get_child (GTK_POPOVER (popover));
 
-  /**
-   * At the moment, the popover stack surely contains only
-   * one child of type GtkMenuSectionBox, which contains
-   * a single GtkBox.
-   */
-  menu_section = stack_children->data;
-  menu_section_children = gtk_container_get_children (GTK_CONTAINER (menu_section));
+  /* GtkViewport */
+  widget = gtk_scrolled_window_get_child (GTK_SCROLLED_WINDOW (widget));
+
+  /* GtkStack */
+  popover_stack = gtk_viewport_get_child (GTK_VIEWPORT (widget));
+
+  menu_section = gtk_widget_get_first_child (popover_stack);
 
 	/**
 	 * Get the unique box's children.
 	 */
-  menu_section_box = menu_section_children->data;
-  menu_section_box_children = gtk_container_get_children (GTK_CONTAINER (menu_section_box));
+  menu_section_box = gtk_widget_get_first_child (menu_section);
 
-  gtk_style_context_add_class (gtk_widget_get_style_context (menu_section_box), "calendars-list");
+  gtk_widget_add_css_class (menu_section_box, "calendars-list");
 
   /**
    * Iterate through the GtkModelButtons inside the menu section box.
    */
-  for (aux = menu_section_box_children; aux != NULL; aux = aux->next)
+  for (child = gtk_widget_get_first_child (menu_section_box);
+       child;
+       child = gtk_widget_get_next_sibling (child))
     {
-      GtkWidget *button_box;
-      GList *button_box_children, *aux2;
-
-      button_box = gtk_bin_get_child (GTK_BIN (aux->data));
-      button_box_children = gtk_container_get_children (GTK_CONTAINER (button_box));
+      GtkWidget *child2;
 
       /**
        * Since there is no guarantee that the first child is
        * the GtkImage we're looking for, we have to iterate
        * through the children and check if the types match.
        */
-      for (aux2 = button_box_children; aux2 != NULL; aux2 = aux2->next)
+      for (child2 = gtk_widget_get_first_child (child);
+           child2;
+           child2 = gtk_widget_get_next_sibling (child2))
         {
-          GtkWidget *button_box_child;
-          button_box_child = aux2->data;
-
-          if (g_type_is_a (G_OBJECT_TYPE (button_box_child), GTK_TYPE_IMAGE))
+          if (GTK_IS_IMAGE (child2))
             {
-              gtk_style_context_add_class (gtk_widget_get_style_context (button_box_child), "calendar-color-image");
-              gtk_widget_show (button_box_child);
+              gtk_widget_add_css_class (child2, "calendar-color-image");
+              gtk_widget_set_margin_end (child2, 12);
+              gtk_widget_show (child2);
               break;
             }
         }
-
-      g_list_free (button_box_children);
     }
-
-  g_list_free (stack_children);
-  g_list_free (menu_section_children);
-  g_list_free (menu_section_box_children);
 }
 
 /**
@@ -858,7 +878,7 @@ get_alarm_trigger_minutes (GcalEvent          *event,
 /**
  * should_change_date_for_scroll:
  * @scroll_value: the current scroll value
- * @scroll_event: the #GdkEventScroll that is being parsed
+ * @scroll_event: the #GdkEvent that is being parsed
  *
  * Utility function to check if the date should change based
  * on the scroll. The date is changed when the user scrolls
@@ -868,12 +888,14 @@ get_alarm_trigger_minutes (GcalEvent          *event,
  * Returns: %TRUE if the date should change, %FALSE otherwise.
  */
 gboolean
-should_change_date_for_scroll (gdouble        *scroll_value,
-                               GdkEventScroll *scroll_event)
+should_change_date_for_scroll (gdouble  *scroll_value,
+                               GdkEvent *scroll_event)
 {
-  gdouble delta_y;
+  gdouble dx, dy;
 
-  switch (scroll_event->direction)
+  g_return_val_if_fail (gdk_event_get_event_type (scroll_event) == GDK_SCROLL, FALSE);
+
+  switch (gdk_scroll_event_get_direction (scroll_event))
     {
     case GDK_SCROLL_DOWN:
       *scroll_value = SCROLL_HARDNESS;
@@ -884,8 +906,8 @@ should_change_date_for_scroll (gdouble        *scroll_value,
       break;
 
     case GDK_SCROLL_SMOOTH:
-      gdk_event_get_scroll_deltas ((GdkEvent*) scroll_event, NULL, &delta_y);
-      *scroll_value += delta_y;
+      gdk_scroll_event_get_deltas (scroll_event, &dx, &dy);
+      *scroll_value += dy;
       break;
 
     /* Ignore horizontal scrolling for now */
@@ -922,86 +944,6 @@ is_source_enabled (ESource *source)
   return e_source_selectable_get_selected (selectable);
 }
 
-/**
- * ask_recurrence_modification_type:
- * @parent: a #GtkWidget
- * @modtype: an #ECalObjModType
- * @source: an #ESource
- *
- * Assigns the appropriate modtype while modifying an event
- * based on user's choice in the GtkMessageDialog that pops up.
- * The modtype helps the user choose the part of recurrent events
- * to modify. Such as Only This Event, Subsequent events
- * or All events.
- *
- * Returns: %TRUE if user chooses appropriate option and
- * @modtype is assigned, %FALSE otherwise.
- */
-gboolean
-ask_recurrence_modification_type (GtkWidget             *parent,
-                                  GcalRecurrenceModType *modtype,
-                                  GcalCalendar          *calendar)
-{
-  GtkDialogFlags flags;
-  ECalClient *client;
-  GtkWidget *dialog;
-  gboolean is_set;
-  gint result;
-
-  flags = GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT;
-  *modtype = GCAL_RECURRENCE_MOD_THIS_ONLY;
-
-  dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (parent)),
-                                   flags,
-                                   GTK_MESSAGE_QUESTION,
-                                   GTK_BUTTONS_NONE,
-                                   _("The event you are trying to modify is recurring. The changes you have selected should be applied to:"));
-
-  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                          _("_Cancel"),
-                          GTK_RESPONSE_CANCEL,
-                          _("_Only This Event"),
-                          GTK_RESPONSE_ACCEPT,
-                          NULL);
-
-  client = gcal_calendar_get_client (calendar);
-
-  if (!e_client_check_capability (E_CLIENT (client), E_CAL_STATIC_CAPABILITY_NO_THISANDFUTURE))
-    gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Subsequent events"), GTK_RESPONSE_OK);
-
-  gtk_dialog_add_button (GTK_DIALOG (dialog), _("_All events"), GTK_RESPONSE_YES);
-
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtk_widget_get_toplevel (parent)));
-
-  result = gtk_dialog_run (GTK_DIALOG (dialog));
-
-  switch (result)
-    {
-      case GTK_RESPONSE_CANCEL:
-        is_set = FALSE;
-        break;
-      case GTK_RESPONSE_ACCEPT:
-        *modtype = GCAL_RECURRENCE_MOD_THIS_ONLY;
-        is_set = TRUE;
-        break;
-      case GTK_RESPONSE_OK:
-        *modtype = GCAL_RECURRENCE_MOD_THIS_AND_FUTURE;
-        is_set = TRUE;
-        break;
-      case GTK_RESPONSE_YES:
-        *modtype = GCAL_RECURRENCE_MOD_ALL;
-        is_set = TRUE;
-        break;
-      default:
-        is_set = FALSE;
-        break;
-    }
-
-  gtk_widget_destroy (GTK_WIDGET (dialog));
-
-  return is_set;
-}
-
 struct
 {
   const gchar        *territory;
@@ -1082,29 +1024,30 @@ is_workday (guint day)
 }
 
 GList*
-filter_event_list_by_uid_and_modtype (GList                 *widgets,
-                                      GcalRecurrenceModType  mod,
-                                      const gchar           *uid)
+filter_children_by_uid_and_modtype (GtkWidget             *widget,
+                                    GcalRecurrenceModType  mod,
+                                    const gchar           *uid)
 {
+  GtkWidget *child;
   GcalEvent *event;
   GList *result;
-  GList *l;
 
   event = NULL;
   result = NULL;
 
   /* First pass: find the GcalEvent */
-  for (l = widgets; l != NULL; l = l->next)
+  for (child = gtk_widget_get_first_child (widget);
+       child;
+       child = gtk_widget_get_next_sibling (child))
     {
       GcalEventWidget *event_widget;
       GcalEvent *ev;
 
-      event_widget = l->data;
-
       /* Safeguard against stray widgets */
-      if (!GCAL_IS_EVENT_WIDGET (event_widget))
+      if (!GCAL_IS_EVENT_WIDGET (child))
         continue;
 
+      event_widget = GCAL_EVENT_WIDGET (child);
       ev = gcal_event_widget_get_event (event_widget);
 
       /*
@@ -1131,17 +1074,18 @@ filter_event_list_by_uid_and_modtype (GList                 *widgets,
       id = e_cal_component_get_id (component);
       id_prefix = g_strdup_printf ("%s:%s", gcal_calendar_get_id (calendar), e_cal_component_id_get_uid (id));
 
-      for (l = widgets; l != NULL; l = l->next)
+      for (child = gtk_widget_get_first_child (widget);
+           child;
+           child = gtk_widget_get_next_sibling (child))
         {
           GcalEventWidget *event_widget;
           GcalEvent *ev;
 
-          event_widget = l->data;
-
           /* Safeguard against stray widgets */
-          if (!GCAL_IS_EVENT_WIDGET (event_widget))
+          if (!GCAL_IS_EVENT_WIDGET (child))
             continue;
 
+          event_widget = GCAL_EVENT_WIDGET (child);
           ev = gcal_event_widget_get_event (event_widget);
 
           if (g_str_equal (gcal_event_get_uid (ev), uid))
@@ -1168,38 +1112,19 @@ filter_event_list_by_uid_and_modtype (GList                 *widgets,
   return result;
 }
 
-gboolean
-gcal_translate_child_window_position (GtkWidget *target,
-                                      GdkWindow *child_window,
-                                      gdouble    src_x,
-                                      gdouble    src_y,
-                                      gdouble   *real_x,
-                                      gdouble   *real_y)
+static GDBusProxy *
+create_dbus_proxy (GDBusConnection *connection,
+                   const gchar     *name,
+                   const gchar     *object_path)
 {
-  GdkWindow *window;
-  gdouble x, y;
-
-  x = src_x;
-  y = src_y;
-
-  /* Find the (x, y) values relative to the workbench */
-  window = child_window;
-  while (window && window != gtk_widget_get_window (target))
-    {
-      gdk_window_coords_to_parent (window, x, y, &x, &y);
-      window = gdk_window_get_parent (window);
-    }
-
-  if (!window)
-    return FALSE;
-
-  if (real_x)
-    *real_x = x;
-
-  if (real_y)
-    *real_y = y;
-
-  return TRUE;
+  return g_dbus_proxy_new_sync (connection,
+                                G_DBUS_PROXY_FLAGS_NONE,
+                                NULL,
+                                name,
+                                object_path,
+                                "org.gtk.Actions",
+                                NULL,
+                                NULL);
 }
 
 void
@@ -1233,14 +1158,11 @@ gcal_utils_launch_online_accounts_panel (GDBusConnection *connection,
   params[1] = g_variant_new_array (G_VARIANT_TYPE ("v"), array, 1);
   params[2] = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
 
-  proxy = g_dbus_proxy_new_sync (connection,
-                                 G_DBUS_PROXY_FLAGS_NONE,
-                                 NULL,
-                                 "org.gnome.ControlCenter",
-                                 "/org/gnome/ControlCenter",
-                                 "org.gtk.Actions",
-                                 NULL,
-                                 NULL);
+  proxy = create_dbus_proxy (connection, "org.gnome.Settings", "/org/gnome/Settings");
+
+  /* Fallback for old GNOME versions */
+  if (!proxy)
+    proxy = create_dbus_proxy (connection, "org.gnome.ControlCenter", "/org/gnome/ControlCenter");
 
   if (!proxy)
     {
@@ -1332,3 +1254,124 @@ out:
   if (out_meeting_url)
     *out_meeting_url = g_steal_pointer (&meeting_url);
 }
+
+typedef struct
+{
+  GcalEvent                 *event;
+  GcalAskRecurrenceCallback  callback;
+  gpointer                   user_data;
+} AskRecurrenceData;
+
+static void
+on_message_dialog_response_cb (GtkDialog         *dialog,
+                               gint               response,
+                               AskRecurrenceData *data)
+{
+  GcalRecurrenceModType mod_type;
+
+  switch (response)
+    {
+      case GTK_RESPONSE_CANCEL:
+        mod_type = GCAL_RECURRENCE_MOD_NONE;
+        break;
+      case GTK_RESPONSE_ACCEPT:
+        mod_type = GCAL_RECURRENCE_MOD_THIS_ONLY;
+        break;
+      case GTK_RESPONSE_OK:
+        mod_type = GCAL_RECURRENCE_MOD_THIS_AND_FUTURE;
+        break;
+      case GTK_RESPONSE_YES:
+        mod_type = GCAL_RECURRENCE_MOD_ALL;
+        break;
+      default:
+        mod_type = GCAL_RECURRENCE_MOD_NONE;
+        break;
+    }
+
+  gtk_window_destroy (GTK_WINDOW (dialog));
+
+  data->callback (data->event, mod_type, data->user_data);
+  g_clear_object (&data->event);
+  g_clear_pointer (&data, g_free);
+}
+
+void
+gcal_utils_ask_recurrence_modification_type (GtkWidget                 *parent,
+                                             GcalEvent                 *event,
+                                             gboolean                   show_mod_all,
+                                             GcalAskRecurrenceCallback  callback,
+                                             gpointer                   user_data)
+{
+  AskRecurrenceData *data;
+  GtkDialogFlags flags;
+  ECalClient *client;
+  GtkWidget *dialog;
+
+  data = g_new0 (AskRecurrenceData, 1);
+  data->event = g_object_ref (event);
+  data->callback = callback;
+  data->user_data = user_data;
+
+  flags = GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT;
+
+  dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_native (parent)),
+                                   flags,
+                                   GTK_MESSAGE_QUESTION,
+                                   GTK_BUTTONS_NONE,
+                                   _("The event you are trying to modify is recurring. The changes you have selected should be applied to:"));
+
+  gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+                          _("_Cancel"),
+                          GTK_RESPONSE_CANCEL,
+                          _("_Only This Event"),
+                          GTK_RESPONSE_ACCEPT,
+                          NULL);
+
+  client = gcal_calendar_get_client (gcal_event_get_calendar (event));
+
+  if (!e_client_check_capability (E_CLIENT (client), E_CAL_STATIC_CAPABILITY_NO_THISANDFUTURE))
+    gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Subsequent events"), GTK_RESPONSE_OK);
+
+  if (show_mod_all)
+    gtk_dialog_add_button (GTK_DIALOG (dialog), _("_All events"), GTK_RESPONSE_YES);
+
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gtk_widget_get_native (parent)));
+  g_signal_connect (dialog, "response", G_CALLBACK (on_message_dialog_response_cb), data);
+
+  gtk_window_present (GTK_WINDOW (dialog));
+}
+
+/**
+ * gcal_util_translate_time_string:
+ * @str: String to translate
+ *
+ * Translate @str according to the locale defined by LC_TIME; unlike
+ * dcgettext(), the translations is still taken from the LC_MESSAGES
+ * catalogue and not the LC_TIME one.
+ *
+ * Returns: the translated string
+ */
+const gchar *
+gcal_util_translate_time_string (const gchar *str)
+{
+  const gchar *locale = g_getenv ("LC_TIME");
+  const gchar *res;
+  gchar *sep;
+  locale_t old_loc;
+  locale_t loc = (locale_t) 0;
+
+  if (locale)
+    loc = newlocale (LC_MESSAGES_MASK, locale, (locale_t) 0);
+
+  old_loc = uselocale (loc);
+
+  sep = strchr (str, '\004');
+  res = g_dpgettext (NULL, str, sep ? sep - str + 1 : 0);
+
+  uselocale (old_loc);
+
+  if (loc != (locale_t) 0)
+    freelocale (loc);
+
+  return res;
+}
